@@ -78,7 +78,9 @@ enum vkd3d_shader_meta_flags
     VKD3D_SHADER_META_FLAG_EXPORTS_SAMPLE_MASK = 1 << 17,
     VKD3D_SHADER_META_FLAG_FORCE_PRE_RASTERIZATION_BEFORE_DISPATCH = 1 << 18,
     VKD3D_SHADER_META_FLAG_FORCE_GRAPHICS_BEFORE_DISPATCH = 1 << 19,
-    VKD3D_SHADER_META_FLAG_USES_DEPTH_STENCIL_WRITE = 1 << 20,
+    VKD3D_SHADER_META_FLAG_FORCE_COMPUTE_BARRIER_BEFORE_DISPATCH = 1 << 20,
+    VKD3D_SHADER_META_FLAG_USES_DEPTH_STENCIL_WRITE = 1 << 21,
+    VKD3D_SHADER_META_FLAG_DISABLE_OPTIMIZATIONS = 1 << 22
 };
 
 struct vkd3d_shader_meta
@@ -227,6 +229,9 @@ enum vkd3d_shader_interface_flag
     VKD3D_SHADER_INTERFACE_DESCRIPTOR_QA_BUFFER             = 0x00000010u,
     /* In this model, use descriptor_size_cbv_srv_uav as array stride for raw VA buffer. */
     VKD3D_SHADER_INTERFACE_RAW_VA_ALIAS_DESCRIPTOR_BUFFER   = 0x00000020u,
+    VKD3D_SHADER_INTERFACE_INSTRUCTION_QA_BUFFER            = 0x00000040u,
+    VKD3D_SHADER_INTERFACE_INSTRUCTION_QA_BUFFER_FULL       = 0x00000080u,
+    VKD3D_SHADER_INTERFACE_INSTRUCTION_QA_BUFFER_FLUSH_NAN  = 0x00000100u,
 };
 
 struct vkd3d_shader_stage_io_entry
@@ -269,10 +274,10 @@ struct vkd3d_shader_interface_info
     const struct vkd3d_shader_descriptor_binding *offset_buffer_binding;
 
 #ifdef VKD3D_ENABLE_DESCRIPTOR_QA
-    /* Ignored unless VKD3D_SHADER_INTERFACE_DESCRIPTOR_QA_BUFFER is set. */
-    const struct vkd3d_shader_descriptor_binding *descriptor_qa_global_binding;
-    /* Ignored unless VKD3D_SHADER_INTERFACE_DESCRIPTOR_QA_BUFFER is set. */
-    const struct vkd3d_shader_descriptor_binding *descriptor_qa_heap_binding;
+    /* Ignored unless VKD3D_SHADER_INTERFACE_{DESCRIPTOR,INSTRUCTION}_QA_BUFFER is set. */
+    const struct vkd3d_shader_descriptor_binding *descriptor_qa_payload_binding;
+    /* Ignored unless VKD3D_SHADER_INTERFACE_{DESCRIPTOR,INSTRUCTION}_QA_BUFFER is set. */
+    const struct vkd3d_shader_descriptor_binding *descriptor_qa_control_binding;
 #endif
 
     const struct vkd3d_shader_stage_io_map *stage_input_map;
@@ -373,9 +378,14 @@ enum vkd3d_shader_target_extension
     VKD3D_SHADER_TARGET_EXTENSION_BARYCENTRIC_KHR,
     VKD3D_SHADER_TARGET_EXTENSION_MIN_PRECISION_IS_NATIVE_16BIT,
     VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_FP16_DENORM_PRESERVE,
+    VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_FP32_DENORM_FLUSH,
     VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_FP64_DENORM_PRESERVE,
+    VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_FP16_INF_NAN_PRESERVE,
+    VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_FP32_INF_NAN_PRESERVE,
+    VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_FP64_INF_NAN_PRESERVE,
     VKD3D_SHADER_TARGET_EXTENSION_SUPPORT_SUBGROUP_PARTITIONED_NV,
     VKD3D_SHADER_TARGET_EXTENSION_COMPUTE_SHADER_DERIVATIVES_NV,
+    VKD3D_SHADER_TARGET_EXTENSION_COMPUTE_SHADER_DERIVATIVES_KHR,
     VKD3D_SHADER_TARGET_EXTENSION_QUAD_CONTROL_RECONVERGENCE,
     VKD3D_SHADER_TARGET_EXTENSION_RAW_ACCESS_CHAINS_NV,
     VKD3D_SHADER_TARGET_EXTENSION_COUNT,
@@ -431,16 +441,32 @@ enum vkd3d_shader_quirk
     /* Driver workarounds. Force loops to not be unrolled with SPIR-V control masks. */
     VKD3D_SHADER_QUIRK_FORCE_LOOP = (1 << 13),
 
-    /* Requests META_FLAG_FORCE_COMPUTE_BARRIER_AFTER_DISPATCH to be set in shader meta. */
+    /* Requests META_FLAG_FORCE_COMPUTE_BARRIER_{AFTER,BEFORE}_DISPATCH to be set in shader meta. */
     VKD3D_SHADER_QUIRK_FORCE_COMPUTE_BARRIER = (1 << 14),
+    VKD3D_SHADER_QUIRK_FORCE_PRE_COMPUTE_BARRIER = (1 << 15),
 
     /* Range check every descriptor heap access with dynamic index and robustness check it. */
-    VKD3D_SHADER_QUIRK_DESCRIPTOR_HEAP_ROBUSTNESS = (1 << 15),
+    VKD3D_SHADER_QUIRK_DESCRIPTOR_HEAP_ROBUSTNESS = (1 << 16),
 
     /* Requests META_FLAG_FORCE_PRE_RASTERIZATION_BEFORE_DISPATCH to be set in shader meta. */
-    VKD3D_SHADER_QUIRK_FORCE_PRE_RASTERIZATION_BARRIER = (1 << 16),
+    VKD3D_SHADER_QUIRK_FORCE_PRE_RASTERIZATION_BARRIER = (1 << 17),
     /* Requests META_FLAG_FORCE_GRAPHICS_BEFORE_DISPATCH to be set in shader meta. */
-    VKD3D_SHADER_QUIRK_FORCE_GRAPHICS_BARRIER = (1 << 17),
+    VKD3D_SHADER_QUIRK_FORCE_GRAPHICS_BARRIER = (1 << 18),
+
+    /* VK_PIPELINE_CREATE_DISABLE_OPTIMIZATIONS. For driver workarounds where optimizations break stuff. */
+    VKD3D_SHADER_QUIRK_DISABLE_OPTIMIZATIONS = (1 << 19),
+
+    /* Forces NoContract on every expression that can take it (only applies to VS).
+     * Useful as a global quirk when games are shipping broken code. */
+    VKD3D_SHADER_QUIRK_FORCE_NOCONTRACT_MATH_VS = (1 << 20),
+
+    /* Works around cases where app is relying on coherency between threads in a workgroup,
+     * but forgets to use Device memory barriers properly. */
+    VKD3D_SHADER_QUIRK_FORCE_DEVICE_MEMORY_BARRIER_THREAD_GROUP_COHERENCY = (1 << 21),
+
+    /* Extremely specific workaround for cubemap importance sampling pass.
+     * The lowest res mips may contain garbage. */
+    VKD3D_SHADER_QUIRK_ASSUME_BROKEN_SUB_8x8_CUBE_MIPS = (1 << 22)
 };
 
 struct vkd3d_shader_quirk_hash
