@@ -557,10 +557,10 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
         const struct vkd3d_shader_interface_info *shader_interface_info,
         const struct vkd3d_shader_compile_arguments *compiler_args)
 {
+    dxil_spv_option_compute_shader_derivatives compute_shader_derivatives = {{ DXIL_SPV_OPTION_COMPUTE_SHADER_DERIVATIVES }};
     dxil_spv_option_denorm_preserve_support denorm_preserve = {{ DXIL_SPV_OPTION_DENORM_PRESERVE_SUPPORT }};
     uint32_t wave_size_min, wave_size_max, wave_size_preferred;
     struct vkd3d_dxil_remap_userdata remap_userdata;
-    bool compute_shader_derivatives = false;
     unsigned int raw_va_binding_count = 0;
     unsigned int num_root_descriptors = 0;
     unsigned int root_constant_words = 0;
@@ -577,8 +577,8 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
 
     dxil_spv_set_thread_log_callback(vkd3d_dxil_log_callback, NULL);
 
+    hash = spirv->meta.hash == 0 ? vkd3d_shader_hash(dxbc) : spirv->meta.hash;
     memset(&spirv->meta, 0, sizeof(spirv->meta));
-    hash = vkd3d_shader_hash(dxbc);
     spirv->meta.hash = hash;
 
     /* Cannot replace mesh shaders until we have reflected the IO layout. */
@@ -595,6 +595,8 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
         spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_PRE_RASTERIZATION_BEFORE_DISPATCH;
     if (quirks & VKD3D_SHADER_QUIRK_FORCE_GRAPHICS_BARRIER)
         spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_GRAPHICS_BEFORE_DISPATCH;
+    if (quirks & VKD3D_SHADER_QUIRK_DISABLE_OPTIMIZATIONS)
+        spirv->meta.flags |= VKD3D_SHADER_META_FLAG_DISABLE_OPTIMIZATIONS;
 
     dxil_spv_begin_thread_allocator_context();
 
@@ -720,10 +722,10 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
         helper.enabled = DXIL_SPV_TRUE;
         helper.version = DXIL_SPV_DESCRIPTOR_QA_INTERFACE_VERSION;
         helper.shader_hash = hash;
-        helper.global_desc_set = shader_interface_info->descriptor_qa_global_binding->set;
-        helper.global_binding = shader_interface_info->descriptor_qa_global_binding->binding;
-        helper.heap_desc_set = shader_interface_info->descriptor_qa_heap_binding->set;
-        helper.heap_binding = shader_interface_info->descriptor_qa_heap_binding->binding;
+        helper.global_desc_set = shader_interface_info->descriptor_qa_payload_binding->set;
+        helper.global_binding = shader_interface_info->descriptor_qa_payload_binding->binding;
+        helper.heap_desc_set = shader_interface_info->descriptor_qa_control_binding->set;
+        helper.heap_binding = shader_interface_info->descriptor_qa_control_binding->binding;
 
         if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
         {
@@ -732,6 +734,30 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
             goto end;
         }
     }
+    else if (shader_interface_info->flags & VKD3D_SHADER_INTERFACE_INSTRUCTION_QA_BUFFER)
+    {
+        struct dxil_spv_option_instruction_instrumentation helper;
+        helper.base.type = DXIL_SPV_OPTION_INSTRUCTION_INSTRUMENTATION;
+        helper.enabled = DXIL_SPV_TRUE;
+        helper.version = DXIL_SPV_INSTRUCTION_INSTRUMENTATION_INTERFACE_VERSION;
+        helper.shader_hash = hash;
+        helper.payload_desc_set = shader_interface_info->descriptor_qa_payload_binding->set;
+        helper.payload_binding = shader_interface_info->descriptor_qa_payload_binding->binding;
+        helper.control_desc_set = shader_interface_info->descriptor_qa_control_binding->set;
+        helper.control_binding = shader_interface_info->descriptor_qa_control_binding->binding;
+        helper.type = DXIL_SPV_INSTRUCTION_INSTRUMENTATION_TYPE_EXTERNALLY_VISIBLE_WRITE_NAN_INF;
+        if (shader_interface_info->flags & VKD3D_SHADER_INTERFACE_INSTRUCTION_QA_BUFFER_FULL)
+            helper.type = DXIL_SPV_INSTRUCTION_INSTRUMENTATION_TYPE_FULL_NAN_INF;
+        else if (shader_interface_info->flags & VKD3D_SHADER_INTERFACE_INSTRUCTION_QA_BUFFER_FLUSH_NAN)
+            helper.type = DXIL_SPV_INSTRUCTION_INSTRUMENTATION_TYPE_FLUSH_NAN_TO_ZERO;
+
+        if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
+        {
+            ERR("dxil-spirv does not support INSTRUCTION_INSTRUMENTATION.\n");
+            ret = VKD3D_ERROR_NOT_IMPLEMENTED;
+            goto end;
+        }
+    }
 #endif
 
     if (shader_interface_info->flags & VKD3D_SHADER_INTERFACE_RAW_VA_ALIAS_DESCRIPTOR_BUFFER)
@@ -772,10 +798,16 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
     }
 
     {
+        bool force_nocontract =
+                (quirks & VKD3D_SHADER_QUIRK_FORCE_NOCONTRACT_MATH) ||
+                ((quirks & VKD3D_SHADER_QUIRK_FORCE_NOCONTRACT_MATH_VS) &&
+                shader_interface_info->stage == VK_SHADER_STAGE_VERTEX_BIT);
+
         const struct dxil_spv_option_precise_control helper =
                 { { DXIL_SPV_OPTION_PRECISE_CONTROL },
-                        (quirks & VKD3D_SHADER_QUIRK_FORCE_NOCONTRACT_MATH) ? DXIL_SPV_TRUE : DXIL_SPV_FALSE,
+                        force_nocontract ? DXIL_SPV_TRUE : DXIL_SPV_FALSE,
                         DXIL_SPV_FALSE };
+
         if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
         {
             WARN("dxil-spirv does not support PRECISE_CONTROL.\n");
@@ -897,8 +929,11 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
             }
             else if (compiler_args->target_extensions[i] == VKD3D_SHADER_TARGET_EXTENSION_COMPUTE_SHADER_DERIVATIVES_NV)
             {
-                /* See comments below. */
-                compute_shader_derivatives = true;
+                compute_shader_derivatives.supports_nv = DXIL_SPV_TRUE;
+            }
+            else if (compiler_args->target_extensions[i] == VKD3D_SHADER_TARGET_EXTENSION_COMPUTE_SHADER_DERIVATIVES_KHR)
+            {
+                compute_shader_derivatives.supports_khr = DXIL_SPV_TRUE;
             }
             else if (compiler_args->target_extensions[i] == VKD3D_SHADER_TARGET_EXTENSION_MIN_PRECISION_IS_NATIVE_16BIT)
             {
@@ -1024,17 +1059,11 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
 
     /* For legacy reasons, COMPUTE_SHADER_DERIVATIVES_NV is default true in dxil-spirv,
      * so we have to override it to false as needed. */
+    if (dxil_spv_converter_add_option(converter, &compute_shader_derivatives.base) != DXIL_SPV_SUCCESS)
     {
-        const dxil_spv_option_compute_shader_derivatives_nv helper =
-                { { DXIL_SPV_OPTION_COMPUTE_SHADER_DERIVATIVES_NV },
-                        compute_shader_derivatives ? DXIL_SPV_TRUE : DXIL_SPV_FALSE};
-
-        if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
-        {
-            ERR("dxil-spirv does not support COMPUTE_SHADER_DERIVATIVES_NV.\n");
-            ret = VKD3D_ERROR_NOT_IMPLEMENTED;
-            goto end;
-        }
+        ERR("dxil-spirv does not support COMPUTE_SHADER_DERIVATIVES.\n");
+        ret = VKD3D_ERROR_NOT_IMPLEMENTED;
+        goto end;
     }
 
     if (quirks & VKD3D_SHADER_QUIRK_INVARIANT_POSITION)
@@ -1073,6 +1102,34 @@ int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
         }
     }
 
+    if (quirks & VKD3D_SHADER_QUIRK_FORCE_DEVICE_MEMORY_BARRIER_THREAD_GROUP_COHERENCY)
+    {
+        const dxil_spv_option_shader_quirk helper =
+                { { DXIL_SPV_OPTION_SHADER_QUIRK },
+                    DXIL_SPV_SHADER_QUIRK_FORCE_DEVICE_MEMORY_BARRIERS_THREAD_GROUP_COHERENCE };
+
+        if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
+        {
+            ERR("dxil-spirv does not support SHADER_QUIRK.\n");
+            ret = VKD3D_ERROR_NOT_IMPLEMENTED;
+            goto end;
+        }
+    }
+
+    if (quirks & VKD3D_SHADER_QUIRK_ASSUME_BROKEN_SUB_8x8_CUBE_MIPS)
+    {
+        const dxil_spv_option_shader_quirk helper =
+                { { DXIL_SPV_OPTION_SHADER_QUIRK },
+                    DXIL_SPV_SHADER_QUIRK_ASSUME_BROKEN_SUB_8x8_CUBE_MIPS };
+
+        if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
+        {
+            ERR("dxil-spirv does not support SHADER_QUIRK.\n");
+            ret = VKD3D_ERROR_NOT_IMPLEMENTED;
+            goto end;
+        }
+    }
+
     remap_userdata.shader_interface_info = shader_interface_info;
     remap_userdata.shader_interface_local_info = NULL;
     remap_userdata.num_root_descriptors = num_root_descriptors;
@@ -1203,10 +1260,14 @@ int vkd3d_shader_compile_dxil_export(const struct vkd3d_shader_code *dxil,
     quirks = vkd3d_shader_compile_arguments_select_quirks(compiler_args, hash);
     if (quirks & VKD3D_SHADER_QUIRK_FORCE_COMPUTE_BARRIER)
         spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_COMPUTE_BARRIER_AFTER_DISPATCH;
+    if (quirks & VKD3D_SHADER_QUIRK_FORCE_PRE_COMPUTE_BARRIER)
+        spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_COMPUTE_BARRIER_BEFORE_DISPATCH;
     if (quirks & VKD3D_SHADER_QUIRK_FORCE_PRE_RASTERIZATION_BARRIER)
         spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_PRE_RASTERIZATION_BEFORE_DISPATCH;
     if (quirks & VKD3D_SHADER_QUIRK_FORCE_GRAPHICS_BARRIER)
         spirv->meta.flags |= VKD3D_SHADER_META_FLAG_FORCE_GRAPHICS_BEFORE_DISPATCH;
+    if (quirks & VKD3D_SHADER_QUIRK_DISABLE_OPTIMIZATIONS)
+        spirv->meta.flags |= VKD3D_SHADER_META_FLAG_DISABLE_OPTIMIZATIONS;
 
     /* For user provided (not mangled) export names, just inherit that name. */
     if (!demangled_export)
@@ -1418,10 +1479,10 @@ int vkd3d_shader_compile_dxil_export(const struct vkd3d_shader_code *dxil,
         helper.enabled = DXIL_SPV_TRUE;
         helper.version = DXIL_SPV_DESCRIPTOR_QA_INTERFACE_VERSION;
         helper.shader_hash = hash;
-        helper.global_desc_set = shader_interface_info->descriptor_qa_global_binding->set;
-        helper.global_binding = shader_interface_info->descriptor_qa_global_binding->binding;
-        helper.heap_desc_set = shader_interface_info->descriptor_qa_heap_binding->set;
-        helper.heap_binding = shader_interface_info->descriptor_qa_heap_binding->binding;
+        helper.global_desc_set = shader_interface_info->descriptor_qa_payload_binding->set;
+        helper.global_binding = shader_interface_info->descriptor_qa_payload_binding->binding;
+        helper.heap_desc_set = shader_interface_info->descriptor_qa_control_binding->set;
+        helper.heap_binding = shader_interface_info->descriptor_qa_control_binding->binding;
 
         if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
         {
@@ -1430,6 +1491,26 @@ int vkd3d_shader_compile_dxil_export(const struct vkd3d_shader_code *dxil,
             goto end;
         }
     }
+    else if (shader_interface_info->flags & VKD3D_SHADER_INTERFACE_INSTRUCTION_QA_BUFFER)
+    {
+        struct dxil_spv_option_instruction_instrumentation helper;
+        helper.base.type = DXIL_SPV_OPTION_INSTRUCTION_INSTRUMENTATION;
+        helper.enabled = DXIL_SPV_TRUE;
+        helper.version = DXIL_SPV_INSTRUCTION_INSTRUMENTATION_INTERFACE_VERSION;
+        helper.shader_hash = hash;
+        helper.payload_desc_set = shader_interface_info->descriptor_qa_payload_binding->set;
+        helper.payload_binding = shader_interface_info->descriptor_qa_payload_binding->binding;
+        helper.control_desc_set = shader_interface_info->descriptor_qa_control_binding->set;
+        helper.control_binding = shader_interface_info->descriptor_qa_control_binding->binding;
+        helper.type = DXIL_SPV_INSTRUCTION_INSTRUMENTATION_TYPE_EXTERNALLY_VISIBLE_WRITE_NAN_INF;
+
+        if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
+        {
+            ERR("dxil-spirv does not support INSTRUCTION_INSTRUMENTATION.\n");
+            ret = VKD3D_ERROR_NOT_IMPLEMENTED;
+            goto end;
+        }
+    }
 #endif
 
     {
@@ -1537,6 +1618,34 @@ int vkd3d_shader_compile_dxil_export(const struct vkd3d_shader_code *dxil,
         }
     }
 
+    if (quirks & VKD3D_SHADER_QUIRK_FORCE_DEVICE_MEMORY_BARRIER_THREAD_GROUP_COHERENCY)
+    {
+        const dxil_spv_option_shader_quirk helper =
+                { { DXIL_SPV_OPTION_SHADER_QUIRK },
+                    DXIL_SPV_SHADER_QUIRK_FORCE_DEVICE_MEMORY_BARRIERS_THREAD_GROUP_COHERENCE };
+
+        if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
+        {
+            ERR("dxil-spirv does not support SHADER_QUIRK.\n");
+            ret = VKD3D_ERROR_NOT_IMPLEMENTED;
+            goto end;
+        }
+    }
+
+    if (quirks & VKD3D_SHADER_QUIRK_ASSUME_BROKEN_SUB_8x8_CUBE_MIPS)
+    {
+        const dxil_spv_option_shader_quirk helper =
+                { { DXIL_SPV_OPTION_SHADER_QUIRK },
+                    DXIL_SPV_SHADER_QUIRK_ASSUME_BROKEN_SUB_8x8_CUBE_MIPS };
+
+        if (dxil_spv_converter_add_option(converter, &helper.base) != DXIL_SPV_SUCCESS)
+        {
+            ERR("dxil-spirv does not support SHADER_QUIRK.\n");
+            ret = VKD3D_ERROR_NOT_IMPLEMENTED;
+            goto end;
+        }
+    }
+
     if (compiler_args)
     {
         for (i = 0; i < compiler_args->target_extension_count; i++)
