@@ -114,6 +114,7 @@ enum VKD3D_SHADER_INSTRUCTION_HANDLER
     VKD3DSIH_DCL_INTERFACE,
     VKD3DSIH_DCL_OUTPUT,
     VKD3DSIH_DCL_OUTPUT_CONTROL_POINT_COUNT,
+    VKD3DSIH_DCL_OUTPUT_SGV,
     VKD3DSIH_DCL_OUTPUT_SIV,
     VKD3DSIH_DCL_OUTPUT_TOPOLOGY,
     VKD3DSIH_DCL_RESOURCE_RAW,
@@ -435,11 +436,14 @@ enum vkd3d_shader_sync_flags
 {
     VKD3DSSF_THREAD_GROUP        = 0x1,
     VKD3DSSF_GROUP_SHARED_MEMORY = 0x2,
+    VKD3DSSF_UAV_MEMORY_LOCAL    = 0x4,
+    VKD3DSSF_UAV_MEMORY_GLOBAL   = 0x8,
 };
 
 enum vkd3d_shader_uav_flags
 {
     VKD3DSUF_GLOBALLY_COHERENT = 0x2,
+    VKD3DSUF_RASTERIZER_ORDERED = 0x4,
     VKD3DSUF_ORDER_PRESERVING_COUNTER = 0x100,
 };
 
@@ -454,6 +458,7 @@ enum vkd3d_tessellator_domain
 #define VKD3DSI_INDEXED_DYNAMIC         0x4
 #define VKD3DSI_RESINFO_RCP_FLOAT       0x1
 #define VKD3DSI_RESINFO_UINT            0x2
+#define VKD3DSI_RESINFO_MASK            0x3
 #define VKD3DSI_SAMPLE_INFO_UINT        0x1
 #define VKD3DSI_SAMPLER_COMPARISON_MODE 0x1
 
@@ -488,7 +493,7 @@ enum vkd3d_shader_conditional_op
 #define VKD3D_SM5_DS  0x0004u
 #define VKD3D_SM5_CS  0x0005u
 
-#define MAX_IMMEDIATE_CONSTANT_BUFFER_SIZE 4096
+#define MAX_IMMEDIATE_CONSTANT_BUFFER_DWORDS 16384
 #define MAX_REG_OUTPUT 32
 
 enum vkd3d_shader_type
@@ -514,7 +519,7 @@ struct vkd3d_shader_version
 struct vkd3d_shader_immediate_constant_buffer
 {
     unsigned int vec4_count;
-    uint32_t data[MAX_IMMEDIATE_CONSTANT_BUFFER_SIZE];
+    uint32_t data[MAX_IMMEDIATE_CONSTANT_BUFFER_DWORDS];
 };
 
 struct vkd3d_shader_indexable_temp
@@ -761,8 +766,6 @@ static inline bool vkd3d_shader_register_is_output(const struct vkd3d_shader_reg
     return reg->type == VKD3DSPR_OUTPUT || reg->type == VKD3DSPR_COLOROUT;
 }
 
-void vkd3d_shader_trace(void *data);
-
 const char *shader_get_type_prefix(enum vkd3d_shader_type type);
 
 void *shader_sm4_init(const DWORD *byte_code, size_t byte_code_size,
@@ -783,6 +786,8 @@ int shader_parse_input_signature(const void *dxbc, size_t dxbc_length,
         struct vkd3d_shader_signature *signature);
 int shader_parse_output_signature(const void *dxbc, size_t dxbc_length,
         struct vkd3d_shader_signature *signature);
+int shader_parse_patch_constant_signature(const void *dxbc, size_t dxbc_length,
+        struct vkd3d_shader_signature *signature);
 
 struct vkd3d_dxbc_compiler;
 
@@ -912,10 +917,13 @@ static inline unsigned int vkd3d_compact_swizzle(unsigned int swizzle, unsigned
 
 unsigned int vkd3d_shader_scan_get_register_flags(const struct vkd3d_shader_scan_info *scan_info,
         enum vkd3d_shader_register_type type, unsigned int id);
+unsigned int vkd3d_shader_scan_get_idxtemp_components(const struct vkd3d_shader_scan_info *scan_info,
+        const struct vkd3d_shader_register *reg);
 
 /* DXIL support */
 int vkd3d_shader_compile_dxil(const struct vkd3d_shader_code *dxbc,
         struct vkd3d_shader_code *spirv,
+        struct vkd3d_shader_code_debug *spirv_debug,
         const struct vkd3d_shader_interface_info *shader_interface_info,
         const struct vkd3d_shader_compile_arguments *compiler_args);
 
