@@ -599,21 +599,24 @@ void vkd3d_breadcrumb_tracer_report_device_lost(struct vkd3d_breadcrumb_tracer *
     if (device->vk_info.NV_device_diagnostic_checkpoints)
     {
         /* vkGetQueueCheckpointDataNV does not require us to synchronize access to the queue. */
-        queue_family_info = d3d12_device_get_vkd3d_queue_family(device, D3D12_COMMAND_LIST_TYPE_DIRECT);
+        queue_family_info = d3d12_device_get_vkd3d_queue_family(device, D3D12_COMMAND_LIST_TYPE_DIRECT, VK_QUEUE_FAMILY_IGNORED);
+
         for (i = 0; i < queue_family_info->queue_count; i++)
         {
             vk_queue = queue_family_info->queues[i]->vk_queue;
             vkd3d_breadcrumb_tracer_report_queue_nv(tracer, device, vk_queue);
         }
 
-        queue_family_info = d3d12_device_get_vkd3d_queue_family(device, D3D12_COMMAND_LIST_TYPE_COMPUTE);
+        queue_family_info = d3d12_device_get_vkd3d_queue_family(device, D3D12_COMMAND_LIST_TYPE_COMPUTE, VK_QUEUE_FAMILY_IGNORED);
+
         for (i = 0; i < queue_family_info->queue_count; i++)
         {
             vk_queue = queue_family_info->queues[i]->vk_queue;
             vkd3d_breadcrumb_tracer_report_queue_nv(tracer, device, vk_queue);
         }
 
-        queue_family_info = d3d12_device_get_vkd3d_queue_family(device, D3D12_COMMAND_LIST_TYPE_COPY);
+        queue_family_info = d3d12_device_get_vkd3d_queue_family(device, D3D12_COMMAND_LIST_TYPE_COPY, VK_QUEUE_FAMILY_IGNORED);
+
         for (i = 0; i < queue_family_info->queue_count; i++)
         {
             vk_queue = queue_family_info->queues[i]->vk_queue;
@@ -838,12 +841,8 @@ void vkd3d_breadcrumb_tracer_end_command_list(struct d3d12_command_list *list)
 
 void vkd3d_breadcrumb_tracer_update_barrier_hashes(struct vkd3d_breadcrumb_tracer *tracer)
 {
-    vkd3d_shader_hash_t lo_hash;
-    vkd3d_shader_hash_t hi_hash;
     char env[VKD3D_PATH_MAX];
-    uint32_t new_count;
-    char line[64];
-    char *end_ptr;
+    size_t new_count;
     FILE *file;
 
     if (vkd3d_get_env_var("VKD3D_BARRIER_HASHES", env, sizeof(env)))
@@ -852,33 +851,8 @@ void vkd3d_breadcrumb_tracer_update_barrier_hashes(struct vkd3d_breadcrumb_trace
         if (file)
         {
             pthread_mutex_lock(&tracer->barrier_hash_lock);
-            new_count = 0;
-            while (fgets(line, sizeof(line), file))
-            {
-                /* Look for either a single number, or lohash-hihash format. */
-                if (!isalnum(*line))
-                    continue;
-
-                lo_hash = strtoull(line, &end_ptr, 16);
-
-                while (*end_ptr != '\0' && !isalnum(*end_ptr))
-                    end_ptr++;
-
-                hi_hash = strtoull(end_ptr, NULL, 16);
-
-                if (!hi_hash)
-                    hi_hash = lo_hash;
-
-                if (lo_hash || hi_hash)
-                {
-                    vkd3d_array_reserve((void **)&tracer->barrier_hashes, &tracer->barrier_hashes_size,
-                            new_count + 1, sizeof(*tracer->barrier_hashes));
-
-                    tracer->barrier_hashes[new_count].lo = lo_hash;
-                    tracer->barrier_hashes[new_count].hi = hi_hash;
-                    new_count++;
-                }
-            }
+            vkd3d_shader_hash_range_parse(file, &tracer->barrier_hashes, &tracer->barrier_hashes_size,
+                    &new_count, VKD3D_SHADER_HASH_RANGE_KIND_BARRIERS);
             vkd3d_atomic_uint32_store_explicit(&tracer->barrier_hashes_count, new_count, vkd3d_memory_order_relaxed);
             pthread_mutex_unlock(&tracer->barrier_hash_lock);
             fclose(file);
@@ -888,9 +862,10 @@ void vkd3d_breadcrumb_tracer_update_barrier_hashes(struct vkd3d_breadcrumb_trace
     }
 }
 
-bool vkd3d_breadcrumb_tracer_shader_hash_forces_barrier(struct vkd3d_breadcrumb_tracer *tracer, vkd3d_shader_hash_t hash)
+uint32_t vkd3d_breadcrumb_tracer_shader_hash_forces_barrier(struct vkd3d_breadcrumb_tracer *tracer,
+        vkd3d_shader_hash_t hash)
 {
-    bool ret = false;
+    uint32_t flags = 0;
     size_t i;
 
     /* Avoid taking lock every dispatch when we're not explicitly using the feature.
@@ -898,9 +873,10 @@ bool vkd3d_breadcrumb_tracer_shader_hash_forces_barrier(struct vkd3d_breadcrumb_
     if (vkd3d_atomic_uint32_load_explicit(&tracer->barrier_hashes_count, vkd3d_memory_order_relaxed) != 0)
     {
         pthread_mutex_lock(&tracer->barrier_hash_lock);
-        for (i = 0; i < tracer->barrier_hashes_count && !ret; i++)
-            ret = tracer->barrier_hashes[i].lo <= hash && hash <= tracer->barrier_hashes[i].hi;
+        for (i = 0; i < tracer->barrier_hashes_count; i++)
+            if (tracer->barrier_hashes[i].lo <= hash && hash <= tracer->barrier_hashes[i].hi)
+                flags |= tracer->barrier_hashes[i].flags;
         pthread_mutex_unlock(&tracer->barrier_hash_lock);
     }
-    return ret;
+    return flags;
 }
