@@ -150,13 +150,13 @@ static void STDMETHODCALLTYPE d3d12_command_list_SetGraphicsRootSignature_profil
 static void STDMETHODCALLTYPE d3d12_command_list_SetComputeRootDescriptorTable_profiled(d3d12_command_list_iface *iface,
         UINT root_parameter_index, D3D12_GPU_DESCRIPTOR_HANDLE base_descriptor)
 {
-    COMMAND_LIST_PROFILED_CALL(SetComputeRootDescriptorTable, iface, root_parameter_index, base_descriptor);
+    COMMAND_LIST_PROFILED_CALL(SetComputeRootDescriptorTable_default, iface, root_parameter_index, base_descriptor);
 }
 
 static void STDMETHODCALLTYPE d3d12_command_list_SetGraphicsRootDescriptorTable_profiled(d3d12_command_list_iface *iface,
         UINT root_parameter_index, D3D12_GPU_DESCRIPTOR_HANDLE base_descriptor)
 {
-    COMMAND_LIST_PROFILED_CALL(SetGraphicsRootDescriptorTable, iface, root_parameter_index, base_descriptor);
+    COMMAND_LIST_PROFILED_CALL(SetGraphicsRootDescriptorTable_default, iface, root_parameter_index, base_descriptor);
 }
 
 static void STDMETHODCALLTYPE d3d12_command_list_SetComputeRoot32BitConstant_profiled(d3d12_command_list_iface *iface,
@@ -464,7 +464,27 @@ static void STDMETHODCALLTYPE d3d12_command_list_DispatchMesh_profiled(d3d12_com
     COMMAND_LIST_PROFILED_CALL(DispatchMesh, iface, x, y, z);
 }
 
-static CONST_VTBL struct ID3D12GraphicsCommandList6Vtbl d3d12_command_list_vtbl_profiled =
+static void STDMETHODCALLTYPE d3d12_command_list_Barrier_profiled(d3d12_command_list_iface *iface, UINT32 NumBarrierGroups, const D3D12_BARRIER_GROUP *pBarrierGroups)
+{
+    COMMAND_LIST_PROFILED_CALL(Barrier, iface, NumBarrierGroups, pBarrierGroups);
+}
+
+static void STDMETHODCALLTYPE d3d12_command_list_OMSetFrontAndBackStencilRef_profiled(d3d12_command_list_iface *iface, UINT FrontStencilRef, UINT BackStencilRef)
+{
+    COMMAND_LIST_PROFILED_CALL(OMSetFrontAndBackStencilRef, iface, FrontStencilRef, BackStencilRef);
+}
+
+static void STDMETHODCALLTYPE d3d12_command_list_RSSetDepthBias_profiled(d3d12_command_list_iface *iface, FLOAT DepthBias, FLOAT DepthBiasClamp, FLOAT SlopeScaledDepthBias)
+{
+    COMMAND_LIST_PROFILED_CALL(RSSetDepthBias, iface, DepthBias, DepthBiasClamp, SlopeScaledDepthBias);
+}
+
+static void STDMETHODCALLTYPE d3d12_command_list_IASetIndexBufferStripCutValue_profiled(d3d12_command_list_iface *iface, D3D12_INDEX_BUFFER_STRIP_CUT_VALUE IBStripCutValue)
+{
+    COMMAND_LIST_PROFILED_CALL(IASetIndexBufferStripCutValue, iface, IBStripCutValue);
+}
+
+static CONST_VTBL struct ID3D12GraphicsCommandList9Vtbl d3d12_command_list_vtbl_profiled =
 {
     /* IUnknown methods */
     d3d12_command_list_QueryInterface,
@@ -557,6 +577,13 @@ static CONST_VTBL struct ID3D12GraphicsCommandList6Vtbl d3d12_command_list_vtbl_
     d3d12_command_list_RSSetShadingRateImage_profiled,
     /* ID3D12GraphicsCommandList6 methods */
     d3d12_command_list_DispatchMesh_profiled,
+    /* ID3D12GraphicsCommandList7 methods */
+    d3d12_command_list_Barrier_profiled,
+    /* ID3D12GraphicsCommandList8 methods */
+    d3d12_command_list_OMSetFrontAndBackStencilRef_profiled,
+    /* ID3D12GraphicsCommandList9 methods */
+    d3d12_command_list_RSSetDepthBias_profiled,
+    d3d12_command_list_IASetIndexBufferStripCutValue_profiled,
 };
 
 #endif
