@@ -144,8 +144,8 @@ HRESULT vkd3d_memory_transfer_queue_init(struct vkd3d_memory_transfer_queue *que
     memset(queue, 0, sizeof(*queue));
 
     queue->device = device;
-    queue->vkd3d_queue = d3d12_device_allocate_vkd3d_queue(device,
-            device->queue_families[VKD3D_QUEUE_FAMILY_INTERNAL_COMPUTE]);
+    queue->vkd3d_queue = d3d12_device_allocate_vkd3d_queue(
+            device->queue_families[VKD3D_QUEUE_FAMILY_INTERNAL_COMPUTE], NULL);
 
     queue->last_known_value = VKD3D_MEMORY_TRANSFER_COMMAND_BUFFER_COUNT;
     queue->next_signal_value = VKD3D_MEMORY_TRANSFER_COMMAND_BUFFER_COUNT + 1;
@@ -275,22 +275,20 @@ static HRESULT vkd3d_memory_transfer_queue_flush_locked(struct vkd3d_memory_tran
 {
     const struct vkd3d_vk_device_procs *vk_procs = &queue->device->vk_procs;
     const struct vkd3d_subresource_layout *subresource_layout;
+    VkSemaphoreSubmitInfo signal_semaphore_infos[2];
+    VkImageSubresourceLayers vk_subresource_layers;
     VkCopyBufferToImageInfo2 buffer_to_image_copy;
-    struct vkd3d_queue_family_info *queue_family;
-    VkSemaphoreSubmitInfo signal_semaphore_info;
     VkCommandBufferSubmitInfo cmd_buffer_info;
-    struct vkd3d_format_footprint footprint;
     VkCommandBufferBeginInfo begin_info;
     VkImageMemoryBarrier2 image_barrier;
     VkImageSubresource vk_subresource;
-    uint32_t queue_mask, queue_index;
     VkBufferImageCopy2 copy_region;
     VkCommandBuffer vk_cmd_buffer;
     VkDeviceSize buffer_offset;
     VkDependencyInfo dep_info;
     VkSubmitInfo2 submit_info;
+    VkExtent3D mip_extent;
     bool need_transition;
-    uint32_t plane_idx;
     VkQueue vk_queue;
     VkResult vr;
     size_t i;
@@ -366,8 +364,8 @@ static HRESULT vkd3d_memory_transfer_queue_flush_locked(struct vkd3d_memory_tran
                 }
 
                 vk_subresource = d3d12_resource_get_vk_subresource(transfer->resource, transfer->subresource_idx, false);
-                plane_idx = transfer->subresource_idx / d3d12_resource_desc_get_sub_resource_count_per_plane(&transfer->resource->desc);
-                footprint = vkd3d_format_footprint_for_plane(transfer->resource->format, plane_idx);
+                vk_subresource_layers = vk_subresource_layers_from_subresource(&vk_subresource);
+                mip_extent = d3d12_resource_desc_get_vk_subresource_extent(&transfer->resource->desc, transfer->resource->format, &vk_subresource_layers);
 
                 subresource_layout = &transfer->resource->subresource_layouts[transfer->subresource_idx];
                 buffer_offset = subresource_layout->offset + vkd3d_format_get_data_offset(transfer->resource->format,
@@ -376,9 +374,9 @@ static HRESULT vkd3d_memory_transfer_queue_flush_locked(struct vkd3d_memory_tran
                 copy_region.sType = VK_STRUCTURE_TYPE_BUFFER_IMAGE_COPY_2;
                 copy_region.pNext = NULL;
                 copy_region.bufferOffset = transfer->resource->mem.offset + buffer_offset;
-                copy_region.bufferRowLength = d3d12_resource_desc_get_width(&transfer->resource->desc, vk_subresource.mipLevel + footprint.subsample_x_log2);
-                copy_region.bufferImageHeight = d3d12_resource_desc_get_height(&transfer->resource->desc, vk_subresource.mipLevel + footprint.subsample_y_log2);
-                copy_region.imageSubresource = vk_subresource_layers_from_subresource(&vk_subresource);
+                copy_region.bufferRowLength = mip_extent.width;
+                copy_region.bufferImageHeight = mip_extent.height;
+                copy_region.imageSubresource = vk_subresource_layers;
                 copy_region.imageOffset = transfer->offset;
                 copy_region.imageExtent = transfer->extent;
 
@@ -411,18 +409,24 @@ static HRESULT vkd3d_memory_transfer_queue_flush_locked(struct vkd3d_memory_tran
     cmd_buffer_info.sType = VK_STRUCTURE_TYPE_COMMAND_BUFFER_SUBMIT_INFO;
     cmd_buffer_info.commandBuffer = vk_cmd_buffer;
 
-    memset(&signal_semaphore_info, 0, sizeof(signal_semaphore_info));
-    signal_semaphore_info.sType = VK_STRUCTURE_TYPE_SEMAPHORE_SUBMIT_INFO;
-    signal_semaphore_info.semaphore = queue->vk_semaphore;
-    signal_semaphore_info.value = queue->next_signal_value;
-    signal_semaphore_info.stageMask = VK_PIPELINE_STAGE_2_COPY_BIT;
+    memset(&signal_semaphore_infos, 0, sizeof(signal_semaphore_infos));
+    signal_semaphore_infos[0].sType = VK_STRUCTURE_TYPE_SEMAPHORE_SUBMIT_INFO;
+    signal_semaphore_infos[0].semaphore = queue->vk_semaphore;
+    signal_semaphore_infos[0].value = queue->next_signal_value;
+    signal_semaphore_infos[0].stageMask = VK_PIPELINE_STAGE_2_COPY_BIT;
+
+    /* External submission */
+    signal_semaphore_infos[1].sType = VK_STRUCTURE_TYPE_SEMAPHORE_SUBMIT_INFO;
+    signal_semaphore_infos[1].semaphore = queue->vkd3d_queue->submission_timeline;
+    signal_semaphore_infos[1].value = ++queue->vkd3d_queue->submission_timeline_count;
+    signal_semaphore_infos[1].stageMask = VK_PIPELINE_STAGE_2_COPY_BIT;
 
     memset(&submit_info, 0, sizeof(submit_info));
     submit_info.sType = VK_STRUCTURE_TYPE_SUBMIT_INFO_2;
     submit_info.commandBufferInfoCount = 1;
     submit_info.pCommandBufferInfos = &cmd_buffer_info;
-    submit_info.signalSemaphoreInfoCount = 1;
-    submit_info.pSignalSemaphoreInfos = &signal_semaphore_info;
+    submit_info.signalSemaphoreInfoCount = ARRAY_SIZE(signal_semaphore_infos);
+    submit_info.pSignalSemaphoreInfos = signal_semaphore_infos;
 
     vr = VK_CALL(vkQueueSubmit2(vk_queue, 1, &submit_info, VK_NULL_HANDLE));
     vkd3d_queue_release(queue->vkd3d_queue);
@@ -436,21 +440,7 @@ static HRESULT vkd3d_memory_transfer_queue_flush_locked(struct vkd3d_memory_tran
     }
 
     /* Stall future submissions on other queues until the clear has finished */
-    queue_mask = queue->device->unique_queue_mask;
-
-    while (queue_mask)
-    {
-        queue_index = vkd3d_bitmask_iter32(&queue_mask);
-        queue_family = queue->device->queue_families[queue_index];
-
-        for (i = 0; i < queue_family->queue_count; i++)
-        {
-            vkd3d_queue_add_wait(queue_family->queues[i],
-                    NULL,
-                    queue->vk_semaphore,
-                    queue->next_signal_value, 0);
-        }
-    }
+    vkd3d_add_wait_to_all_queues(queue->device, queue->vk_semaphore, queue->next_signal_value);
 
     /* Keep next_signal always one ahead of the last signaled value */
     queue->next_signal_value += 1;
@@ -636,7 +626,9 @@ static D3D12_HEAP_TYPE vkd3d_normalize_heap_type(const D3D12_HEAP_PROPERTIES *he
         case D3D12_CPU_PAGE_PROPERTY_WRITE_BACK:
             return D3D12_HEAP_TYPE_READBACK;
         case D3D12_CPU_PAGE_PROPERTY_WRITE_COMBINE:
-            return D3D12_HEAP_TYPE_UPLOAD;
+            return heap_properties->MemoryPoolPreference == D3D12_MEMORY_POOL_L1
+                ? D3D12_HEAP_TYPE_GPU_UPLOAD
+                : D3D12_HEAP_TYPE_UPLOAD;
         default:
             break;
     }
@@ -655,6 +647,18 @@ static HRESULT vkd3d_select_memory_flags(struct d3d12_device *device, const D3D1
 
         case D3D12_HEAP_TYPE_UPLOAD:
             *type_flags = device->memory_info.upload_heap_memory_properties;
+            if (vkd3d_atomic_uint32_load_explicit(&device->memory_info.has_used_gpu_upload_heap, vkd3d_memory_order_relaxed) == 1)
+            {
+                *type_flags &= ~VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
+                if (vkd3d_config_flags & VKD3D_CONFIG_FLAG_LOG_MEMORY_BUDGET)
+                {
+                    INFO("UPLOAD_HEAP memory will not use ReBar memory because the game has allocated memory from the GPU_UPLOAD_HEAP in the past.\n");
+                }
+            }
+            break;
+
+        case D3D12_HEAP_TYPE_GPU_UPLOAD:
+            *type_flags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT | VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT;
             break;
 
         case D3D12_HEAP_TYPE_READBACK:
@@ -672,8 +676,11 @@ static HRESULT vkd3d_select_memory_flags(struct d3d12_device *device, const D3D1
                     break;
                 case D3D12_CPU_PAGE_PROPERTY_WRITE_COMBINE:
                     *type_flags = VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT;
+
                     if (vkd3d_config_flags & VKD3D_CONFIG_FLAG_FORCE_HOST_CACHED)
                         *type_flags |= VK_MEMORY_PROPERTY_HOST_CACHED_BIT;
+                    else if (heap_properties->MemoryPoolPreference == D3D12_MEMORY_POOL_L1)
+                        *type_flags |= VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
                     break;
                 case D3D12_CPU_PAGE_PROPERTY_NOT_AVAILABLE:
                     *type_flags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
@@ -797,29 +804,42 @@ static HRESULT vkd3d_try_allocate_device_memory(struct d3d12_device *device,
     const VkPhysicalDeviceMemoryProperties *memory_props = &device->memory_properties;
     const struct vkd3d_vk_device_procs *vk_procs = &device->vk_procs;
     struct vkd3d_memory_info *memory_info = &device->memory_info;
+    uint32_t type_mask, device_local_mask, candidate_mask;
     VkMemoryAllocateInfo allocate_info;
     VkDeviceSize *type_current;
-    uint32_t type_mask;
     bool rebar_budget;
     VkResult vr;
 
+    device_local_mask = 0u;
+    candidate_mask = 0u;
+
     type_mask = base_type_mask;
-    allocate_info.sType = VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO;
-    allocate_info.pNext = pNext;
-    allocate_info.allocationSize = size;
-    allocate_info.memoryTypeIndex = UINT32_MAX;
 
     while (type_mask)
     {
         uint32_t type_index = vkd3d_bitmask_iter32(&type_mask);
+
         if ((memory_props->memoryTypes[type_index].propertyFlags & type_flags) == type_flags)
         {
-            allocate_info.memoryTypeIndex = type_index;
-            break;
+            candidate_mask |= 1u << type_index;
+
+            if (memory_props->memoryTypes[type_index].propertyFlags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)
+                device_local_mask |= 1u << type_index;
         }
     }
 
-    if (allocate_info.memoryTypeIndex == UINT32_MAX)
+    /* If device-local memory is not explicitly requested and the type mask covers system memory,
+     * ensure that we don't accidentally use HVV. This may happen in case a driver does not expose
+     * uncached system memory. */
+    if (!(type_flags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT) && (candidate_mask != device_local_mask))
+        candidate_mask &= ~device_local_mask;
+
+    allocate_info.sType = VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO;
+    allocate_info.pNext = pNext;
+    allocate_info.allocationSize = size;
+    allocate_info.memoryTypeIndex = vkd3d_bitmask_tzcnt32(candidate_mask);
+
+    if (allocate_info.memoryTypeIndex >= memory_props->memoryTypeCount)
     {
         /* We consider CACHED optional here if we cannot find a memory type that supports it.
          * Failing to allocate CACHED is not a scenario where we would fall back. */
@@ -978,7 +998,7 @@ HRESULT vkd3d_allocate_device_memory(struct d3d12_device *device,
              * E.g. there exists no memory type that is not DEVICE_LOCAL and covers both RT and DS.
              * For this case, we have no choice but to not allocate,
              * and defer actual memory allocation to CreatePlacedResource() time.
-			 * NVIDIA bug reference for fixing this case: 2175829. */
+             * NVIDIA bug reference for fixing this case: 2175829. */
             WARN("Memory allocation failed, but it is not possible to fallback to system memory here. Deferring allocation.\n");
             return hr;
         }
@@ -1055,6 +1075,9 @@ static void *vkd3d_allocate_write_watch_pointer(const D3D12_HEAP_PROPERTIES *pro
     case D3D12_HEAP_TYPE_UPLOAD:
         protect = PAGE_READWRITE | PAGE_WRITECOMBINE;
         break;
+    case D3D12_HEAP_TYPE_GPU_UPLOAD:
+        protect = PAGE_READWRITE | PAGE_WRITECOMBINE;
+        break;
     case D3D12_HEAP_TYPE_READBACK:
         /* WRITE_WATCH fails for this type in native D3D12,
          * otherwise it would be PAGE_READWRITE. */
@@ -1252,6 +1275,25 @@ static HRESULT vkd3d_memory_allocation_init(struct vkd3d_memory_allocation *allo
         }
     }
 
+    /* Custom heaps have been resolved to regular heap types earlier. */
+    assert(allocation->heap_type != D3D12_HEAP_TYPE_CUSTOM);
+
+    if (allocation->heap_type == D3D12_HEAP_TYPE_GPU_UPLOAD)
+    {
+        if (!device->memory_info.has_gpu_upload_heap)
+        {
+            ERR("Trying to allocate memory on GPU_UPLOAD_HEAP which is not supported on current device.\n");
+            return E_INVALIDARG;
+        }
+
+        if (!(info->flags & VKD3D_ALLOCATION_FLAG_INTERNAL_SCRATCH) &&
+            vkd3d_atomic_uint32_exchange_explicit(&device->memory_info.has_used_gpu_upload_heap, 1, vkd3d_memory_order_relaxed) != 1 &&
+            (vkd3d_config_flags & VKD3D_CONFIG_FLAG_LOG_MEMORY_BUDGET))
+        {
+            INFO("Allocated memory on GPU_UPLOAD_HEAP, disabling automatic UPLOAD to ReBar promotion.\n");
+        }
+    }
+
     if (device->device_info.memory_priority_features.memoryPriority &&
         (type_flags & VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT))
     {
@@ -1274,7 +1316,7 @@ static HRESULT vkd3d_memory_allocation_init(struct vkd3d_memory_allocation *allo
     else
     {
         hr = vkd3d_allocate_device_memory(device, memory_requirements.size, type_flags,
-                type_mask, &flags_info, true, &allocation->device_allocation);
+                type_mask, &flags_info, allocation->heap_type == D3D12_HEAP_TYPE_UPLOAD, &allocation->device_allocation);
     }
 
     if (FAILED(hr))
@@ -1618,7 +1660,7 @@ void vkd3d_memory_allocator_cleanup(struct vkd3d_memory_allocator *allocator, st
 static HRESULT vkd3d_memory_allocator_try_add_chunk(struct vkd3d_memory_allocator *allocator, struct d3d12_device *device,
         const D3D12_HEAP_PROPERTIES *heap_properties, D3D12_HEAP_FLAGS heap_flags, uint32_t type_mask,
         VkMemoryPropertyFlags optional_properties,
-        VkBufferUsageFlags explicit_global_buffer_usage,
+        VkBufferUsageFlags2KHR explicit_global_buffer_usage,
         VkDeviceSize minimum_size,
         struct vkd3d_memory_chunk **chunk)
 {
@@ -1634,6 +1676,7 @@ static HRESULT vkd3d_memory_allocator_try_add_chunk(struct vkd3d_memory_allocato
     alloc_info.flags = VKD3D_ALLOCATION_FLAG_NO_FALLBACK;
     alloc_info.optional_memory_properties = optional_properties;
     alloc_info.vk_memory_priority = vkd3d_convert_to_vk_prio(D3D12_RESIDENCY_PRIORITY_NORMAL);
+    alloc_info.explicit_global_buffer_usage = 0;
 
     if (minimum_size < VKD3D_VA_BLOCK_SIZE)
         alloc_info.memory_requirements.size = VKD3D_MEMORY_CHUNK_SIZE;
@@ -1667,7 +1710,7 @@ static HRESULT vkd3d_memory_allocator_try_suballocate_memory(struct vkd3d_memory
         struct d3d12_device *device, const VkMemoryRequirements *memory_requirements, uint32_t type_mask,
         VkMemoryPropertyFlags optional_properties,
         const D3D12_HEAP_PROPERTIES *heap_properties, D3D12_HEAP_FLAGS heap_flags,
-        VkBufferUsageFlags explicit_global_buffer_usage,
+        VkBufferUsageFlags2KHR explicit_global_buffer_usage,
         struct vkd3d_memory_allocation *allocation)
 {
     const D3D12_HEAP_FLAGS heap_flag_mask = ~(D3D12_HEAP_FLAG_CREATE_NOT_ZEROED |
@@ -1771,9 +1814,12 @@ static HRESULT vkd3d_suballocate_memory(struct d3d12_device *device, struct vkd3
     return hr;
 }
 
-static inline bool vkd3d_driver_implicitly_clears(VkDriverId driver_id)
+static inline bool vkd3d_driver_implicitly_clears(struct d3d12_device *device)
 {
-    switch (driver_id)
+    if (device->workarounds.amdgpu_broken_clearvram)
+        return false;
+
+    switch (device->device_info.vulkan_1_2_properties.driverID)
     {
         /* Known to pass test_stress_suballocation which hits this path. */
         case VK_DRIVER_ID_MESA_RADV:
@@ -1802,8 +1848,7 @@ bool vkd3d_allocate_image_memory_prefers_dedicated(struct d3d12_device *device,
     /* If we don't need to sub-allocate, and we don't need to clear any buffers
      * there is no need to allocate a GLOBAL_BUFFER. */
     return requirements->size >= VKD3D_VA_BLOCK_SIZE &&
-            (vkd3d_driver_implicitly_clears(device->device_info.vulkan_1_2_properties.driverID) ||
-                    (heap_flags & D3D12_HEAP_FLAG_CREATE_NOT_ZEROED));
+            (vkd3d_driver_implicitly_clears(device) || (heap_flags & D3D12_HEAP_FLAG_CREATE_NOT_ZEROED));
 }
 
 static bool vkd3d_memory_info_allow_suballocate(struct d3d12_device *device,
@@ -1836,7 +1881,7 @@ static bool vkd3d_memory_info_allow_suballocate(struct d3d12_device *device,
     /* We may or may not want to disable this workaround if EXT_pageable is supported,
      * but it's good to not have two different allocation paths on NVIDIA and RADV for now. */
 
-    if (is_cpu_accessible_heap(&info->heap_properties) ||
+    if (is_cpu_accessible_system_memory_heap(&info->heap_properties) ||
             !(info->flags & VKD3D_ALLOCATION_FLAG_ALLOW_IMAGE_SUBALLOCATION))
     {
         return false;
@@ -1861,9 +1906,7 @@ HRESULT vkd3d_allocate_memory(struct d3d12_device *device, struct vkd3d_memory_a
      * This is relying on implementation details.
      * RADV definitely does this, and it seems like NV also does it.
      * TODO: an extension for this would be nice. */
-    implementation_implicitly_clears =
-            vkd3d_driver_implicitly_clears(device->device_info.vulkan_1_2_properties.driverID) &&
-            !suballocate;
+    implementation_implicitly_clears = vkd3d_driver_implicitly_clears(device) && !suballocate;
 
     needs_clear = !implementation_implicitly_clears &&
             !(info->heap_flags & D3D12_HEAP_FLAG_CREATE_NOT_ZEROED) &&
@@ -1912,7 +1955,7 @@ static bool vkd3d_heap_allocation_accept_deferred_resource_placements(struct d3d
 
     /* Only accept deferrals for DEFAULT / CPU_NOT_AVAILABLE heaps.
      * If we're going for host memory, we have nowhere left to fall back to either way. */
-    if (is_cpu_accessible_heap(heap_properties))
+    if (is_cpu_accessible_system_memory_heap(heap_properties))
         return false;
 
     type_mask = vkd3d_select_memory_types(device, heap_properties, heap_flags);
@@ -1935,6 +1978,7 @@ HRESULT vkd3d_allocate_heap_memory(struct d3d12_device *device, struct vkd3d_mem
     alloc_info.heap_flags = info->heap_desc.Flags;
     alloc_info.host_ptr = info->host_ptr;
     alloc_info.vk_memory_priority = info->vk_memory_priority;
+    alloc_info.explicit_global_buffer_usage = info->explicit_global_buffer_usage;
 
     alloc_info.flags |= info->extra_allocation_flags;
 
@@ -1949,7 +1993,7 @@ HRESULT vkd3d_allocate_heap_memory(struct d3d12_device *device, struct vkd3d_mem
     if (info->heap_desc.Flags & D3D12_HEAP_FLAG_DENY_BUFFERS)
         alloc_info.explicit_global_buffer_usage = VK_BUFFER_USAGE_TRANSFER_DST_BIT;
 
-    if (is_cpu_accessible_heap(&info->heap_desc.Properties))
+    if (is_cpu_accessible_system_memory_heap(&info->heap_desc.Properties))
     {
         if (info->heap_desc.Flags & D3D12_HEAP_FLAG_DENY_BUFFERS)
         {
