@@ -239,6 +239,7 @@ bool vkd3d_enumerate_meta_command_parameters(struct d3d12_device *device, REFGUI
 
 static void d3d12_meta_command_destroy(struct d3d12_meta_command *meta_command)
 {
+    d3d_destruction_notifier_free(&meta_command->destruction_notifier);
     vkd3d_private_store_destroy(&meta_command->private_store);
 
     vkd3d_free(meta_command);
@@ -247,6 +248,8 @@ static void d3d12_meta_command_destroy(struct d3d12_meta_command *meta_command)
 static HRESULT STDMETHODCALLTYPE d3d12_meta_command_QueryInterface(d3d12_meta_command_iface *iface,
         REFIID riid, void **object)
 {
+    struct d3d12_meta_command *meta_command = impl_from_ID3D12MetaCommand(iface);
+
     TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), object);
 
     if (!object)
@@ -263,6 +266,13 @@ static HRESULT STDMETHODCALLTYPE d3d12_meta_command_QueryInterface(d3d12_meta_co
         return S_OK;
     }
 
+    if (IsEqualGUID(riid, &IID_ID3DDestructionNotifier))
+    {
+        ID3DDestructionNotifier_AddRef(&meta_command->destruction_notifier.ID3DDestructionNotifier_iface);
+        *object = &meta_command->destruction_notifier.ID3DDestructionNotifier_iface;
+        return S_OK;
+    }
+
     WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(riid));
 
     *object = NULL;
@@ -418,24 +428,35 @@ static void d3d12_meta_command_exec_dstorage(struct d3d12_meta_command *meta_com
             meta_ops->dstorage.vk_emit_nv_memory_decompression_regions_layout,
             VK_SHADER_STAGE_COMPUTE_BIT, 0, sizeof(push_args), &push_args));
 
+    memset(&dep_info, 0, sizeof(dep_info));
+    dep_info.sType = VK_STRUCTURE_TYPE_DEPENDENCY_INFO;
+    dep_info.memoryBarrierCount = 1;
+    dep_info.pMemoryBarriers = &vk_barrier;
+
+    /* This barrier is pure nonsense and should not be needed, but apparently it is anyway.
+     * FF XVI demo has a pattern of copy -> copy-to-uav barrier -> gdeflate.
+     * We're synchronized with UAV here, so this should work fine, but apparently it doesn't
+     * and this arbitrary COMPUTE -> COMPUTE barrier happens to work around the issue.
+     * I can only deduce this is a driver bug somehow with barrier emission. */
+    memset(&vk_barrier, 0, sizeof(vk_barrier));
+    vk_barrier.sType = VK_STRUCTURE_TYPE_MEMORY_BARRIER_2;
+    vk_barrier.srcStageMask = VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT;
+    vk_barrier.srcAccessMask = VK_ACCESS_2_SHADER_WRITE_BIT;
+    vk_barrier.dstStageMask = VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT;
+    vk_barrier.dstAccessMask = VK_ACCESS_2_SHADER_READ_BIT | VK_ACCESS_2_SHADER_WRITE_BIT;
+    VK_CALL(vkCmdPipelineBarrier2(list->cmd.vk_command_buffer, &dep_info));
+
     workgroup_count = vkd3d_compute_workgroup_count(parameters->stream_count, 32);
     VK_CALL(vkCmdDispatch(list->cmd.vk_command_buffer, workgroup_count, 1, 1));
 
     /* Iterate over individual streams and dispatch another compute shader
      * that emits the actual VkDecompressMemoryRegionNV structures. */
-    memset(&vk_barrier, 0, sizeof(vk_barrier));
-    vk_barrier.sType = VK_STRUCTURE_TYPE_MEMORY_BARRIER_2;
     vk_barrier.srcStageMask = VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT;
     vk_barrier.srcAccessMask = VK_ACCESS_2_SHADER_WRITE_BIT;
     vk_barrier.dstStageMask = VK_PIPELINE_STAGE_2_COMPUTE_SHADER_BIT | VK_PIPELINE_STAGE_2_DRAW_INDIRECT_BIT;
     vk_barrier.dstAccessMask = VK_ACCESS_2_INDIRECT_COMMAND_READ_BIT |
             VK_ACCESS_2_SHADER_WRITE_BIT | VK_ACCESS_2_SHADER_READ_BIT;
 
-    memset(&dep_info, 0, sizeof(dep_info));
-    dep_info.sType = VK_STRUCTURE_TYPE_DEPENDENCY_INFO;
-    dep_info.memoryBarrierCount = 1;
-    dep_info.pMemoryBarriers = &vk_barrier;
-
     VK_CALL(vkCmdPipelineBarrier2(list->cmd.vk_command_buffer, &dep_info));
 
     VK_CALL(vkCmdBindPipeline(list->cmd.vk_command_buffer, VK_PIPELINE_BIND_POINT_COMPUTE,
@@ -474,6 +495,16 @@ static void d3d12_meta_command_exec_dstorage(struct d3d12_meta_command *meta_com
 
     VK_CALL(vkCmdPipelineBarrier2(list->cmd.vk_command_buffer, &dep_info));
 
+    VKD3D_BREADCRUMB_TAG("[Control VA + Size, Input VA + Size, Output VA + Size, Scratch VA + Size, StreamCount]");
+    VKD3D_BREADCRUMB_AUX64(parameters->control_buffer_va);
+    VKD3D_BREADCRUMB_AUX64(parameters->control_buffer_size);
+    VKD3D_BREADCRUMB_AUX64(parameters->input_buffer_va);
+    VKD3D_BREADCRUMB_AUX64(parameters->input_buffer_size);
+    VKD3D_BREADCRUMB_AUX64(parameters->output_buffer_va);
+    VKD3D_BREADCRUMB_AUX64(parameters->output_buffer_size);
+    VKD3D_BREADCRUMB_AUX64(parameters->scratch_buffer_va);
+    VKD3D_BREADCRUMB_AUX64(parameters->scratch_buffer_size);
+    VKD3D_BREADCRUMB_AUX32(parameters->stream_count);
     VKD3D_BREADCRUMB_COMMAND(DSTORAGE);
 }
 
@@ -543,6 +574,8 @@ HRESULT d3d12_meta_command_create(struct d3d12_device *device, REFGUID guid,
         return hr;
     }
 
+    d3d_destruction_notifier_init(&object->destruction_notifier, (IUnknown*)&object->ID3D12MetaCommand_iface);
+
     d3d12_device_add_ref(device);
 
     *meta_command = object;
