@@ -28,7 +28,7 @@
 HRESULT vkd3d_queue_timeline_trace_init(struct vkd3d_queue_timeline_trace *trace, struct d3d12_device *device)
 {
     char env[VKD3D_PATH_MAX];
-    unsigned int i, j;
+    unsigned int i;
 
     if (!vkd3d_get_env_var("VKD3D_QUEUE_PROFILE", env, sizeof(env)))
         return S_OK;
@@ -67,11 +67,6 @@ HRESULT vkd3d_queue_timeline_trace_init(struct vkd3d_queue_timeline_trace *trace
                 vkd3d_get_current_thread_id());
     }
 
-    for (i = 0; i < VKD3D_QUEUE_FAMILY_COUNT; i++)
-        if (device->queue_families[i])
-            for (j = 0; j < device->queue_families[i]->queue_count; j++)
-                device->queue_families[i]->queues[j]->need_virtual_wait_values = true;
-
     trace->active = true;
     return S_OK;
 }
@@ -252,6 +247,26 @@ void vkd3d_queue_timeline_trace_complete_low_latency_sleep(struct vkd3d_queue_ti
     vkd3d_queue_timeline_trace_complete_blocking(trace, cookie, "ID3DLowLatencyDevice::LatencySleep()");
 }
 
+struct vkd3d_queue_timeline_trace_cookie
+vkd3d_queue_timeline_trace_register_sparse(struct vkd3d_queue_timeline_trace *trace, uint32_t num_tiles)
+{
+    struct vkd3d_queue_timeline_trace_cookie cookie = {0};
+    struct vkd3d_queue_timeline_trace_state *state;
+    uint64_t submission_count;
+    if (!trace->active)
+        return cookie;
+
+    cookie.index = vkd3d_queue_timeline_trace_allocate_index(trace, &submission_count);
+    if (!cookie.index)
+        return cookie;
+
+    state = &trace->state[cookie.index];
+    state->type = VKD3D_QUEUE_TIMELINE_TRACE_STATE_TYPE_SUBMISSION;
+    state->start_ts = vkd3d_get_current_time_ns();
+    snprintf(state->desc, sizeof(state->desc), "SPARSE #%"PRIu64" (%u tiles)", submission_count, num_tiles);
+    return cookie;
+}
+
 struct vkd3d_queue_timeline_trace_cookie
 vkd3d_queue_timeline_trace_register_execute(struct vkd3d_queue_timeline_trace *trace,
         ID3D12CommandList * const *command_lists, unsigned int count)
@@ -423,6 +438,13 @@ vkd3d_queue_timeline_trace_register_present_wait(struct vkd3d_queue_timeline_tra
     return vkd3d_queue_timeline_trace_register_generic_op(trace, VKD3D_QUEUE_TIMELINE_TRACE_STATE_TYPE_PRESENT_WAIT, str);
 }
 
+struct vkd3d_queue_timeline_trace_cookie
+vkd3d_queue_timeline_trace_register_generic_region(struct vkd3d_queue_timeline_trace *trace, const char *tag)
+{
+    return vkd3d_queue_timeline_trace_register_generic_op(trace, VKD3D_QUEUE_TIMELINE_TRACE_STATE_TYPE_GENERIC_REGION, tag);
+}
+
+
 struct vkd3d_queue_timeline_trace_cookie
 vkd3d_queue_timeline_trace_register_present_block(struct vkd3d_queue_timeline_trace *trace, uint64_t present_id)
 {
@@ -528,8 +550,8 @@ void vkd3d_queue_timeline_trace_complete_execute(struct vkd3d_queue_timeline_tra
         struct vkd3d_fence_worker *worker,
         struct vkd3d_queue_timeline_trace_cookie cookie)
 {
+    double end_ts, start_submit_ts, start_ts, overhead_start_ts, overhead_end_ts;
     const struct vkd3d_queue_timeline_trace_state *state;
-    double end_ts, start_submit_ts, start_ts;
     unsigned int pid;
     const char *tid;
     double *ts_lock;
@@ -541,6 +563,8 @@ void vkd3d_queue_timeline_trace_complete_execute(struct vkd3d_queue_timeline_tra
     start_ts = (double)(state->start_ts - trace->base_ts) * 1e-3;
     start_submit_ts = (double)(state->start_submit_ts - trace->base_ts) * 1e-3;
     end_ts = (double)(vkd3d_get_current_time_ns() - trace->base_ts) * 1e-3;
+    overhead_start_ts = start_ts + 1e-3 * state->overhead_start_offset;
+    overhead_end_ts = start_ts + 1e-3 * state->overhead_end_offset;
 
     if (worker)
     {
@@ -548,6 +572,9 @@ void vkd3d_queue_timeline_trace_complete_execute(struct vkd3d_queue_timeline_tra
             vkd3d_queue_timeline_trace_flush_instantaneous(trace, worker);
 
         tid = worker->timeline.tid;
+        if (state->type == VKD3D_QUEUE_TIMELINE_TRACE_STATE_TYPE_GENERIC_REGION)
+            tid = "regions";
+
         pid = worker->queue->submission_thread_tid;
 
         if (state->type == VKD3D_QUEUE_TIMELINE_TRACE_STATE_TYPE_SUBMISSION)
@@ -561,13 +588,16 @@ void vkd3d_queue_timeline_trace_complete_execute(struct vkd3d_queue_timeline_tra
                 start_submit_ts = start_ts;
         }
 
-        ts_lock = &worker->timeline.lock_end_gpu_ts;
+        if (state->type != VKD3D_QUEUE_TIMELINE_TRACE_STATE_TYPE_GENERIC_REGION)
+        {
+            ts_lock = &worker->timeline.lock_end_gpu_ts;
 
-        if (start_submit_ts < *ts_lock)
-            start_submit_ts = *ts_lock;
-        if (end_ts < start_submit_ts)
-            end_ts = start_submit_ts;
-        *ts_lock = end_ts;
+            if (start_submit_ts < *ts_lock)
+                start_submit_ts = *ts_lock;
+            if (end_ts < start_submit_ts)
+                end_ts = start_submit_ts;
+            *ts_lock = end_ts;
+        }
 
         fprintf(trace->file, "{ \"name\": \"%s\", \"ph\": \"X\", \"tid\": \"%s\", \"pid\": \"0x%04x\", \"ts\": %f, \"dur\": %f },\n",
                 state->desc, tid, pid, start_submit_ts, end_ts - start_submit_ts);
@@ -578,6 +608,9 @@ void vkd3d_queue_timeline_trace_complete_execute(struct vkd3d_queue_timeline_tra
             fprintf(trace->file,
                     "{ \"name\": \"%s\", \"ph\": \"X\", \"tid\": \"submit\", \"pid\": \"0x%04x\", \"ts\": %f, \"dur\": %f },\n",
                     state->desc, pid, start_ts, start_submit_ts - start_ts);
+            fprintf(trace->file,
+                    "{ \"name\": \"%s\", \"ph\": \"X\", \"tid\": \"overhead\", \"pid\": \"0x%04x\", \"ts\": %f, \"dur\": %f },\n",
+                    state->desc, pid, overhead_start_ts, overhead_end_ts - overhead_start_ts);
         }
     }
 
@@ -594,3 +627,25 @@ void vkd3d_queue_timeline_trace_begin_execute(struct vkd3d_queue_timeline_trace
     state = &trace->state[cookie.index];
     state->start_submit_ts = vkd3d_get_current_time_ns();
 }
+
+void vkd3d_queue_timeline_trace_begin_execute_overhead(struct vkd3d_queue_timeline_trace *trace,
+        struct vkd3d_queue_timeline_trace_cookie cookie)
+{
+    struct vkd3d_queue_timeline_trace_state *state;
+    if (!trace->active || cookie.index == 0)
+        return;
+
+    state = &trace->state[cookie.index];
+    state->overhead_start_offset = vkd3d_get_current_time_ns() - state->start_ts;
+}
+
+void vkd3d_queue_timeline_trace_end_execute_overhead(struct vkd3d_queue_timeline_trace *trace,
+        struct vkd3d_queue_timeline_trace_cookie cookie)
+{
+    struct vkd3d_queue_timeline_trace_state *state;
+    if (!trace->active || cookie.index == 0)
+        return;
+
+    state = &trace->state[cookie.index];
+    state->overhead_end_offset = vkd3d_get_current_time_ns() - state->start_ts;
+}
