@@ -45,9 +45,10 @@
 #if !defined(_WIN32) || defined(VKD3D_FORCE_UTILS_WRAPPER)
 # include "vkd3d_threads.h"
 # include "vkd3d.h"
-# include "vkd3d_sonames.h"
 #endif
 
+#include "vkd3d_sonames.h"
+
 #if !defined(_WIN32)
 #include <dlfcn.h>
 #include <unistd.h>
@@ -370,6 +371,41 @@ static inline void wait_queue_idle_no_event_(unsigned int line, ID3D12Device *de
     ID3D12Fence_Release(fence);
 }
 
+static PFN_vkGetInstanceProcAddr pfn_vkGetInstanceProcAddr;
+static PFN_vkGetDeviceProcAddr pfn_vkGetDeviceProcAddr;
+static inline bool init_vulkan_loader(void)
+{
+#ifdef _WIN32
+    HMODULE hmod;
+#else
+    void *mod;
+#endif
+
+    if (pfn_vkGetInstanceProcAddr)
+        return true;
+
+    if (pfn_vkGetDeviceProcAddr)
+        return true;
+
+#ifdef _WIN32
+    hmod = LoadLibraryA(SONAME_LIBVULKAN);
+    if (!hmod)
+        return false;
+
+    pfn_vkGetInstanceProcAddr = (PFN_vkGetInstanceProcAddr)(void *)GetProcAddress(hmod, "vkGetInstanceProcAddr");
+    pfn_vkGetDeviceProcAddr = (PFN_vkGetDeviceProcAddr)(void *)GetProcAddress(hmod, "vkGetDeviceProcAddr");
+#else
+    mod = dlopen(SONAME_LIBVULKAN, RTLD_LAZY);
+    if (!mod)
+        return false;
+
+    pfn_vkGetInstanceProcAddr = (PFN_vkGetInstanceProcAddr)dlsym(mod, "vkGetInstanceProcAddr");
+    pfn_vkGetDeviceProcAddr = (PFN_vkGetDeviceProcAddr)dlsym(mod, "vkGetDeviceProcAddr");
+#endif
+
+    return pfn_vkGetInstanceProcAddr != NULL;
+}
+
 #if defined(_WIN32) && !defined(VKD3D_FORCE_UTILS_WRAPPER)
 static IUnknown *create_warp_adapter(IDXGIFactory4 *factory)
 {
@@ -537,36 +573,12 @@ static inline bool is_amd_vulkan_device(ID3D12Device *device)
 {
     return false;
 }
-#else
 
-static PFN_vkGetInstanceProcAddr pfn_vkGetInstanceProcAddr;
-static bool init_vulkan_loader(void)
+static inline bool is_vk_device_extension_supported(ID3D12Device *device, const char *ext)
 {
-#ifdef _WIN32
-    HMODULE hmod;
-#else
-    void *mod;
-#endif
-
-    if (pfn_vkGetInstanceProcAddr)
-        return true;
-
-#ifdef _WIN32
-    hmod = LoadLibraryA(SONAME_LIBVULKAN);
-    if (!hmod)
-        return false;
-
-    pfn_vkGetInstanceProcAddr = (PFN_vkGetInstanceProcAddr)GetProcAddress(hmod, "vkGetInstanceProcAddr");
-#else
-    mod = dlopen(SONAME_LIBVULKAN, RTLD_LAZY);
-    if (!mod)
-        return false;
-
-    pfn_vkGetInstanceProcAddr = (PFN_vkGetInstanceProcAddr)dlsym(mod, "vkGetInstanceProcAddr");
-#endif
-
-    return pfn_vkGetInstanceProcAddr != NULL;
+    return false;
 }
+#else
 
 static ID3D12Device *create_device(void)
 {
@@ -608,6 +620,46 @@ static bool get_driver_properties(ID3D12Device *device, VkPhysicalDeviceDriverPr
     return true;
 }
 
+static inline bool is_vk_device_extension_supported(ID3D12Device *device, const char *ext)
+{
+    ID3D12DXVKInteropDevice *dxvk_device = NULL;
+    const char **exts = NULL;
+    bool supported = false;
+    UINT extension_count;
+    unsigned int i;
+
+    if (!init_vulkan_loader())
+        return false;
+
+    if (FAILED(ID3D12Device_QueryInterface(device, &IID_ID3D12DXVKInteropDevice, (void **)&dxvk_device)))
+        goto err;
+
+    if (FAILED(ID3D12DXVKInteropDevice_GetDeviceExtensions(dxvk_device, &extension_count, NULL)))
+        goto err;
+
+    exts = malloc(sizeof(*exts) * extension_count);
+    if (!exts)
+        goto err;
+
+    if (FAILED(ID3D12DXVKInteropDevice_GetDeviceExtensions(dxvk_device, &extension_count, exts)))
+        goto err;
+
+    for (i = 0; i < extension_count; i++)
+    {
+        if (!strcmp(ext, exts[i]))
+        {
+            supported = true;
+            break;
+        }
+    }
+
+err:
+    if (dxvk_device)
+        ID3D12DXVKInteropDevice_Release(dxvk_device);
+    free(exts);
+    return supported;
+}
+
 static inline void init_adapter_info(void)
 {
     VkPhysicalDeviceDriverPropertiesKHR driver_properties;
@@ -729,4 +781,15 @@ static inline void enable_d3d12_debug_layer(int argc, char **argv)
     }
 }
 
+static inline bool device_supports_gpu_upload_heap(ID3D12Device *device)
+{
+    D3D12_FEATURE_DATA_D3D12_OPTIONS16 options16;
+    HRESULT hr;
+    hr = ID3D12Device_CheckFeatureSupport(device, D3D12_FEATURE_D3D12_OPTIONS16, &options16, sizeof(options16));
+    if (hr != S_OK)
+        return false;
+
+    return options16.GPUUploadHeapSupported;
+}
+
 #endif  /* __VKD3D_D3D12_CROSSTEST_H */
