@@ -192,6 +192,7 @@ void test_mesh_shader_rendering(void)
 {
     D3D12_ROOT_SIGNATURE_DESC root_signature_desc;
     ID3D12Resource *srv_resource, *uav_resource;
+    D3D12_FEATURE_DATA_D3D12_OPTIONS9 options9;
     D3D12_FEATURE_DATA_D3D12_OPTIONS7 options7;
     ID3D12GraphicsCommandList6 *command_list6;
     D3D12_ROOT_PARAMETER root_parameters[3];
@@ -210,10 +211,12 @@ void test_mesh_shader_rendering(void)
 #include "shaders/mesh_shader/headers/ms_clip_distance.h"
 #include "shaders/mesh_shader/headers/ms_cull_primitive.h"
 #include "shaders/mesh_shader/headers/ms_culling.h"
+#include "shaders/mesh_shader/headers/ms_derivatives.h"
 #include "shaders/mesh_shader/headers/ms_interface_matching.h"
 #include "shaders/mesh_shader/headers/ms_simple.h"
 #include "shaders/mesh_shader/headers/ms_system_values.h"
 #include "shaders/mesh_shader/headers/ps_culling.h"
+#include "shaders/mesh_shader/headers/ps_derivatives.h"
 #include "shaders/mesh_shader/headers/ps_green.h"
 #include "shaders/mesh_shader/headers/ps_interface_matching.h"
 #include "shaders/mesh_shader/headers/ps_system_values.h"
@@ -226,6 +229,8 @@ void test_mesh_shader_rendering(void)
             {{ D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_PS, { ps_culling_code_dxil, sizeof(ps_culling_code_dxil) } }};
     static const union d3d12_shader_bytecode_subobject ps_system_values_subobject =
             {{ D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_PS, { ps_system_values_code_dxil, sizeof(ps_system_values_code_dxil) } }};
+    static const union d3d12_shader_bytecode_subobject ps_derivatives_subobject =
+            {{ D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_PS, { ps_derivatives_code_dxil, sizeof(ps_derivatives_code_dxil) } }};
     static const union d3d12_shader_bytecode_subobject ms_simple_subobject =
             {{ D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_MS, { ms_simple_code_dxil, sizeof(ms_simple_code_dxil) } }};
     static const union d3d12_shader_bytecode_subobject ms_interface_matching_subobject =
@@ -238,6 +243,8 @@ void test_mesh_shader_rendering(void)
             {{ D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_MS, { ms_cull_primitive_code_dxil, sizeof(ms_cull_primitive_code_dxil) } }};
     static const union d3d12_shader_bytecode_subobject ms_system_values_subobject =
             {{ D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_MS, { ms_system_values_code_dxil, sizeof(ms_system_values_code_dxil) } }};
+    static const union d3d12_shader_bytecode_subobject ms_derivatives_subobject =
+            {{ D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_MS, { ms_derivatives_code_dxil, sizeof(ms_derivatives_code_dxil) } }};
 
     static const union d3d12_rasterizer_subobject rasterizer_subobject =
     {{
@@ -313,6 +320,9 @@ void test_mesh_shader_rendering(void)
     hr = ID3D12Device_CheckFeatureSupport(context.device, D3D12_FEATURE_D3D12_OPTIONS7, &options7, sizeof(options7));
     ok(SUCCEEDED(hr), "OPTIONS7 is not supported by runtime.\n");
 
+    memset(&options9, 0, sizeof(options9));
+    ID3D12Device_CheckFeatureSupport(context.device, D3D12_FEATURE_D3D12_OPTIONS9, &options9, sizeof(options9));
+
     if (options7.MeshShaderTier < D3D12_MESH_SHADER_TIER_1)
     {
         skip("Mesh shaders not supported by device.\n");
@@ -340,12 +350,12 @@ void test_mesh_shader_rendering(void)
 
     hr = ID3D12Device2_CreateCommittedResource(device2, &heap_properties, D3D12_HEAP_FLAG_NONE,
             &resource_desc, D3D12_RESOURCE_STATE_COPY_DEST, NULL, &IID_ID3D12Resource, (void **)&srv_resource);
-    ok(SUCCEEDED(hr), "Failed to create SRV resource, hr %#x.\n");
+    ok(SUCCEEDED(hr), "Failed to create SRV resource, hr %#x.\n", hr);
 
     resource_desc.Flags = D3D12_RESOURCE_FLAG_ALLOW_UNORDERED_ACCESS;
     hr = ID3D12Device2_CreateCommittedResource(device2, &heap_properties, D3D12_HEAP_FLAG_NONE,
             &resource_desc, D3D12_RESOURCE_STATE_COPY_DEST, NULL, &IID_ID3D12Resource, (void **)&uav_resource);
-    ok(SUCCEEDED(hr), "Failed to create UAV resource, hr %#x.\n");
+    ok(SUCCEEDED(hr), "Failed to create UAV resource, hr %#x.\n", hr);
 
     memset(&root_signature_desc, 0, sizeof(root_signature_desc));
     root_signature_desc.NumParameters = 3;
@@ -576,6 +586,36 @@ void test_mesh_shader_rendering(void)
     release_resource_readback(&rb);
     ID3D12PipelineState_Release(pipeline_state);
 
+    if (options9.DerivativesInMeshAndAmplificationShadersSupported)
+    {
+        pipeline_desc.ms = ms_derivatives_subobject;
+        pipeline_desc.ps = ps_derivatives_subobject;
+
+        hr = create_pipeline_state_from_stream(device2, &pipeline_desc, &pipeline_state);
+        ok(hr == S_OK, "Failed to create pipeline, hr %#x.\n", hr);
+
+        reset_command_list(context.list, context.allocator);
+        transition_resource_state(context.list, context.render_target, D3D12_RESOURCE_STATE_COPY_SOURCE, D3D12_RESOURCE_STATE_RENDER_TARGET);
+
+        ID3D12GraphicsCommandList6_ClearRenderTargetView(command_list6, context.rtv, white, 0, NULL);
+        ID3D12GraphicsCommandList6_OMSetRenderTargets(command_list6, 1, &context.rtv, false, NULL);
+        ID3D12GraphicsCommandList6_SetGraphicsRootSignature(command_list6, root_signature);
+        ID3D12GraphicsCommandList6_SetGraphicsRootUnorderedAccessView(command_list6, 1, ID3D12Resource_GetGPUVirtualAddress(uav_resource));
+        ID3D12GraphicsCommandList6_SetPipelineState(command_list6, pipeline_state);
+        ID3D12GraphicsCommandList6_RSSetViewports(command_list6, 1, &context.viewport);
+        ID3D12GraphicsCommandList6_RSSetScissorRects(command_list6, 1, &context.scissor_rect);
+        ID3D12GraphicsCommandList6_DispatchMesh(command_list6, 1, 1, 1);
+
+        transition_resource_state(context.list, context.render_target, D3D12_RESOURCE_STATE_RENDER_TARGET, D3D12_RESOURCE_STATE_COPY_SOURCE);
+        check_sub_resource_uint(context.render_target, 0, context.queue, context.list, 0x24000011, 1);
+
+        ID3D12PipelineState_Release(pipeline_state);
+    }
+    else
+    {
+        skip("Derivatives in mesh and amplification shaders not supported.\n");
+    }
+
     ID3D12Resource_Release(srv_resource);
     ID3D12Resource_Release(uav_resource);
     ID3D12RootSignature_Release(root_signature);
@@ -714,7 +754,7 @@ void test_mesh_shader_execute_indirect(void)
 
     hr = ID3D12Device2_CreateCommittedResource(device2, &heap_properties, D3D12_HEAP_FLAG_NONE,
             &resource_desc, D3D12_RESOURCE_STATE_COPY_DEST, NULL, &IID_ID3D12Resource, (void **)&indirect_buffer);
-    ok(SUCCEEDED(hr), "Failed to create SRV resource, hr %#x.\n");
+    ok(SUCCEEDED(hr), "Failed to create SRV resource, hr %#x.\n", hr);
 
     memset(&root_signature_desc, 0, sizeof(root_signature_desc));
     root_signature_desc.NumParameters = 0;
@@ -899,7 +939,11 @@ void test_mesh_shader_execute_indirect_state(void)
     indirect_argument_desc[2].Type = D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH_MESH;
 
     hr = ID3D12Device2_CreateCommandSignature(device2, &command_signature_desc, root_signature, &IID_ID3D12CommandSignature, (void **)&command_signature);
-    ok(SUCCEEDED(hr), "Failed to create command signature, hr %#x.\n", hr);
+    ok(SUCCEEDED(hr) || hr == E_NOTIMPL, "Failed to create command signature, hr %#x.\n", hr);
+    if (FAILED(hr))
+        command_signature = NULL;
+    if (hr == E_NOTIMPL)
+        skip("DGC is likely not implemented. Skipping test.\n");
 
     pipeline_desc.root_signature.type = D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_ROOT_SIGNATURE;
     pipeline_desc.root_signature.root_signature = root_signature;
@@ -927,24 +971,29 @@ void test_mesh_shader_execute_indirect_state(void)
         indirect_buffer = create_upload_buffer(context.device, sizeof(test_data), test_data);
     }
 
-    for (i = 0; i < ARRAY_SIZE(tests); i++)
+    if (command_signature)
     {
-        ID3D12GraphicsCommandList6_SetGraphicsRootSignature(command_list6, root_signature);
-        ID3D12GraphicsCommandList6_SetPipelineState(command_list6, pipeline_state);
-        set_viewport(&viewport, 0, 0, 1, 1, 0, 1);
-        set_rect(&scissor, 0, 0, 1, 1);
-        ID3D12GraphicsCommandList6_RSSetViewports(command_list6, 1, &viewport);
-        ID3D12GraphicsCommandList6_RSSetScissorRects(command_list6, 1, &scissor);
-        ID3D12GraphicsCommandList6_ExecuteIndirect(command_list6, command_signature,
-            tests[i].indirect_count ? 64 : tests[i].data.indirect_count,
-            indirect_buffer, sizeof(struct test_data) * i,
-            tests[i].indirect_count ? indirect_buffer : NULL,
-            tests[i].indirect_count ? (sizeof(struct test_data) * i + offsetof(struct test_data, indirect_count)) : 0);
+        for (i = 0; i < ARRAY_SIZE(tests); i++)
+        {
+            ID3D12GraphicsCommandList6_SetGraphicsRootSignature(command_list6, root_signature);
+            ID3D12GraphicsCommandList6_SetPipelineState(command_list6, pipeline_state);
+            set_viewport(&viewport, 0, 0, 1, 1, 0, 1);
+            set_rect(&scissor, 0, 0, 1, 1);
+            ID3D12GraphicsCommandList6_RSSetViewports(command_list6, 1, &viewport);
+            ID3D12GraphicsCommandList6_RSSetScissorRects(command_list6, 1, &scissor);
+
+            ID3D12GraphicsCommandList6_ExecuteIndirect(command_list6, command_signature,
+                tests[i].indirect_count ? 64 : tests[i].data.indirect_count,
+                indirect_buffer, sizeof(struct test_data) * i,
+                tests[i].indirect_count ? indirect_buffer : NULL,
+                tests[i].indirect_count ? (sizeof(struct test_data) * i + offsetof(struct test_data, indirect_count)) : 0);
+        }
     }
 
     transition_resource_state(context.list, output, D3D12_RESOURCE_STATE_UNORDERED_ACCESS, D3D12_RESOURCE_STATE_COPY_SOURCE);
     get_buffer_readback_with_command_list(output, DXGI_FORMAT_R32_UINT, &rb, context.queue, context.list);
 
+    if (command_signature)
     {
         uint32_t expected[128] = { 0 };
         for (i = 0; i < ARRAY_SIZE(tests); i++)
@@ -959,7 +1008,8 @@ void test_mesh_shader_execute_indirect_state(void)
 
     ID3D12Resource_Release(output);
     ID3D12PipelineState_Release(pipeline_state);
-    ID3D12CommandSignature_Release(command_signature);
+    if (command_signature)
+        ID3D12CommandSignature_Release(command_signature);
     ID3D12Resource_Release(indirect_buffer);
     ID3D12RootSignature_Release(root_signature);
     ID3D12GraphicsCommandList6_Release(command_list6);
@@ -1103,12 +1153,12 @@ void test_amplification_shader(void)
 
     hr = ID3D12Device2_CreateCommittedResource(device2, &heap_properties, D3D12_HEAP_FLAG_NONE,
             &resource_desc, D3D12_RESOURCE_STATE_COPY_DEST, NULL, &IID_ID3D12Resource, (void **)&srv_resource);
-    ok(SUCCEEDED(hr), "Failed to create SRV resource, hr %#x.\n");
+    ok(SUCCEEDED(hr), "Failed to create SRV resource, hr %#x.\n", hr);
 
     resource_desc.Flags = D3D12_RESOURCE_FLAG_ALLOW_UNORDERED_ACCESS;
     hr = ID3D12Device2_CreateCommittedResource(device2, &heap_properties, D3D12_HEAP_FLAG_NONE,
             &resource_desc, D3D12_RESOURCE_STATE_COPY_DEST, NULL, &IID_ID3D12Resource, (void **)&uav_resource);
-    ok(SUCCEEDED(hr), "Failed to create UAV resource, hr %#x.\n");
+    ok(SUCCEEDED(hr), "Failed to create UAV resource, hr %#x.\n", hr);
 
     memset(&root_signature_desc, 0, sizeof(root_signature_desc));
     root_signature_desc.NumParameters = 2;
@@ -1290,13 +1340,6 @@ void test_amplification_shader_execute_indirect_state(void)
     if (!init_test_context(&context, &desc))
         return;
 
-    if (is_radv_device(context.device))
-    {
-        skip("Task DGC not yet supported by RADV, skipping test.\n");
-        destroy_test_context(&context);
-        return;
-    }
-
     memset(&options7, 0, sizeof(options7));
     hr = ID3D12Device_CheckFeatureSupport(context.device, D3D12_FEATURE_D3D12_OPTIONS7, &options7, sizeof(options7));
     ok(SUCCEEDED(hr), "OPTIONS7 is not supported by runtime.\n");
@@ -1341,7 +1384,11 @@ void test_amplification_shader_execute_indirect_state(void)
     indirect_argument_desc[2].Type = D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH_MESH;
 
     hr = ID3D12Device2_CreateCommandSignature(device2, &command_signature_desc, root_signature, &IID_ID3D12CommandSignature, (void **)&command_signature);
-    ok(SUCCEEDED(hr), "Failed to create command signature, hr %#x.\n", hr);
+    ok(SUCCEEDED(hr) || hr == E_NOTIMPL, "Failed to create command signature, hr %#x.\n", hr);
+    if (FAILED(hr))
+        command_signature = NULL;
+    if (hr == E_NOTIMPL)
+        skip("DGC is likely not implemented. Skipping test.\n");
 
     pipeline_desc.root_signature.type = D3D12_PIPELINE_STATE_SUBOBJECT_TYPE_ROOT_SIGNATURE;
     pipeline_desc.root_signature.root_signature = root_signature;
@@ -1366,49 +1413,65 @@ void test_amplification_shader_execute_indirect_state(void)
         indirect_buffer = create_upload_buffer(context.device, sizeof(test_data), test_data);
     }
 
-    for (i = 0; i < ARRAY_SIZE(tests); i++)
+    if (command_signature)
     {
-        ID3D12GraphicsCommandList6_SetGraphicsRootSignature(command_list6, root_signature);
-        ID3D12GraphicsCommandList6_SetPipelineState(command_list6, pipeline_state);
-        set_viewport(&viewport, 0, 0, 1, 1, 0, 1);
-        set_rect(&scissor, 0, 0, 1, 1);
-        ID3D12GraphicsCommandList6_RSSetViewports(command_list6, 1, &viewport);
-        ID3D12GraphicsCommandList6_RSSetScissorRects(command_list6, 1, &scissor);
-        ID3D12GraphicsCommandList6_ExecuteIndirect(command_list6, command_signature,
-            tests[i].indirect_count ? 64 : tests[i].data.indirect_count,
-            indirect_buffer, sizeof(tests[i].data) * i,
-            tests[i].indirect_count ? indirect_buffer : NULL,
-            tests[i].indirect_count ? (sizeof(tests[i].data) * i + offsetof(struct test_data, indirect_count)) : 0);
+        for (i = 0; i < ARRAY_SIZE(tests); i++)
+        {
+            ID3D12GraphicsCommandList6_SetGraphicsRootSignature(command_list6, root_signature);
+            ID3D12GraphicsCommandList6_SetPipelineState(command_list6, pipeline_state);
+            set_viewport(&viewport, 0, 0, 1, 1, 0, 1);
+            set_rect(&scissor, 0, 0, 1, 1);
+            ID3D12GraphicsCommandList6_RSSetViewports(command_list6, 1, &viewport);
+            ID3D12GraphicsCommandList6_RSSetScissorRects(command_list6, 1, &scissor);
+            ID3D12GraphicsCommandList6_ExecuteIndirect(command_list6, command_signature,
+                tests[i].indirect_count ? 64 : tests[i].data.indirect_count,
+                indirect_buffer, sizeof(tests[i].data) * i,
+                tests[i].indirect_count ? indirect_buffer : NULL,
+                tests[i].indirect_count ? (sizeof(tests[i].data) * i + offsetof(struct test_data, indirect_count)) : 0);
+        }
     }
 
     transition_resource_state(context.list, output, D3D12_RESOURCE_STATE_UNORDERED_ACCESS, D3D12_RESOURCE_STATE_COPY_SOURCE);
     get_buffer_readback_with_command_list(output, DXGI_FORMAT_R32_UINT, &rb, context.queue, context.list);
 
-    for (i = 0; i < ARRAY_SIZE(tests); i++)
+    if (command_signature)
     {
-        uint32_t num_tasks, num_groups_per_task, expected;
-        expected = 0;
-
-        for (j = 0; j < tests[i].data.indirect_count; j++)
+        for (i = 0; i < ARRAY_SIZE(tests); i++)
         {
-            num_tasks = tests[i + j].data.tasks[0] * tests[i + j].data.tasks[1] * tests[i + j].data.tasks[2];
-            num_groups_per_task = tests[i + j].data.mesh_groups[0] * tests[i + j].data.mesh_groups[1] * tests[i + j].data.mesh_groups[2];
-            expected += num_tasks; /* every task increments counter */
-            expected += num_tasks * num_groups_per_task; /* every mesh group increments counter. */
-            expected += num_tasks * num_groups_per_task * tests[i + j].data.prims; /* Every primitive increments counter. */
-        }
+            uint32_t num_tasks, num_groups_per_task, expected;
+            expected = 0;
 
-        ok(get_readback_uint(&rb, i, 0, 0) == expected, "Test %u: expected %u, got %u.\n", i, expected, get_readback_uint(&rb, i, 0, 0));
+            for (j = 0; j < tests[i].data.indirect_count; j++)
+            {
+                num_tasks = tests[i + j].data.tasks[0] * tests[i + j].data.tasks[1] * tests[i + j].data.tasks[2];
+                num_groups_per_task = tests[i + j].data.mesh_groups[0] * tests[i + j].data.mesh_groups[1] * tests[i + j].data.mesh_groups[2];
+                expected += num_tasks; /* every task increments counter */
+                expected += num_tasks * num_groups_per_task; /* every mesh group increments counter. */
+                expected += num_tasks * num_groups_per_task * tests[i + j].data.prims; /* Every primitive increments counter. */
+            }
+
+            if (tests[i].data.indirect_count)
+            {
+                ok(get_readback_uint(&rb, tests[i].uav_offset / sizeof(uint32_t), 0, 0) == expected,
+                        "Test %u: expected %u, got %u.\n", i, expected, get_readback_uint(&rb, tests[i].uav_offset / sizeof(uint32_t), 0, 0));
+            }
+            else
+            {
+                ok(get_readback_uint(&rb, i, 0, 0) == expected, "Test %u: expected %u, got %u.\n",
+                        i, expected, get_readback_uint(&rb, i, 0, 0));
+            }
+        }
     }
 
     release_resource_readback(&rb);
 
     ID3D12Resource_Release(output);
     ID3D12PipelineState_Release(pipeline_state);
-    ID3D12CommandSignature_Release(command_signature);
+    if (command_signature)
+        ID3D12CommandSignature_Release(command_signature);
     ID3D12Resource_Release(indirect_buffer);
     ID3D12RootSignature_Release(root_signature);
     ID3D12GraphicsCommandList6_Release(command_list6);
     ID3D12Device2_Release(device2);
     destroy_test_context(&context);
-}
\ No newline at end of file
+}
