@@ -24,8 +24,15 @@ PFN_D3D12_SERIALIZE_VERSIONED_ROOT_SIGNATURE pfn_D3D12SerializeVersionedRootSign
 PFN_D3D12_CREATE_DEVICE pfn_D3D12CreateDevice;
 PFN_D3D12_ENABLE_EXPERIMENTAL_FEATURES pfn_D3D12EnableExperimentalFeatures;
 PFN_D3D12_GET_DEBUG_INTERFACE pfn_D3D12GetDebugInterface;
+D3D_FEATURE_LEVEL vkd3d_device_feature_level = D3D_FEATURE_LEVEL_11_0;
 const char *vkd3d_test_platform = "other";
 struct vkd3d_test_state_context vkd3d_test_state;
+bool use_warp_device;
+unsigned int use_adapter_idx;
+
+#ifdef _WIN32
+RENDERDOC_API_1_0_0 *renderdoc_api;
+#endif
 
 bool compare_float(float f, float g, int ulps)
 {
@@ -203,8 +210,8 @@ void upload_buffer_data_(unsigned int line, ID3D12Resource *buffer, size_t offse
     ID3D12Device_Release(device);
 }
 
-void upload_texture_data_(unsigned int line, ID3D12Resource *texture,
-        const D3D12_SUBRESOURCE_DATA *data, unsigned int sub_resource_count,
+void upload_texture_data_base_(unsigned int line, ID3D12Resource *texture,
+        const D3D12_SUBRESOURCE_DATA *data, unsigned int first_subresource, unsigned int sub_resource_count,
         ID3D12CommandQueue *queue, ID3D12GraphicsCommandList *command_list)
 {
     D3D12_TEXTURE_COPY_LOCATION dst_location, src_location;
@@ -230,7 +237,7 @@ void upload_texture_data_(unsigned int line, ID3D12Resource *texture,
     hr = ID3D12Resource_GetDevice(texture, &IID_ID3D12Device, (void **)&device);
     ok_(line)(SUCCEEDED(hr), "Failed to get device, hr %#x.\n", hr);
 
-    ID3D12Device_GetCopyableFootprints(device, &resource_desc, 0, sub_resource_count,
+    ID3D12Device_GetCopyableFootprints(device, &resource_desc, first_subresource, sub_resource_count,
             0, layouts, row_counts, row_sizes, &required_size);
 
     upload_buffer = create_upload_buffer_(line, device, required_size, NULL);
@@ -251,7 +258,7 @@ void upload_texture_data_(unsigned int line, ID3D12Resource *texture,
     {
         dst_location.pResource = texture;
         dst_location.Type = D3D12_TEXTURE_COPY_TYPE_SUBRESOURCE_INDEX;
-        dst_location.SubresourceIndex = i;
+        dst_location.SubresourceIndex = i + first_subresource;
 
         src_location.pResource = upload_buffer;
         src_location.Type = D3D12_TEXTURE_COPY_TYPE_PLACED_FOOTPRINT;
@@ -275,6 +282,13 @@ void upload_texture_data_(unsigned int line, ID3D12Resource *texture,
     free(row_sizes);
 }
 
+void upload_texture_data_(unsigned int line, ID3D12Resource *texture,
+    const D3D12_SUBRESOURCE_DATA *data, unsigned int sub_resource_count,
+    ID3D12CommandQueue *queue, ID3D12GraphicsCommandList *command_list)
+{
+    upload_texture_data_base_(line, texture, data, 0, sub_resource_count, queue, command_list);
+}
+
 void init_readback(struct resource_readback *rb, ID3D12Resource *buffer,
         uint64_t buffer_size, uint64_t width, uint64_t height, unsigned int depth, uint64_t row_pitch)
 {
@@ -842,6 +856,9 @@ ID3D12CommandSignature *create_command_signature_(unsigned int line,
         case D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH:
             signature_desc.ByteStride = sizeof(D3D12_DISPATCH_ARGUMENTS);
             break;
+        case D3D12_INDIRECT_ARGUMENT_TYPE_DISPATCH_RAYS:
+            signature_desc.ByteStride = sizeof(D3D12_DISPATCH_RAYS_DESC);
+            break;
         default:
             return NULL;
     }
@@ -858,6 +875,7 @@ ID3D12CommandSignature *create_command_signature_(unsigned int line,
 
 bool init_compute_test_context_(unsigned int line, struct test_context *context)
 {
+    D3D12_COMMAND_LIST_TYPE command_list_type = D3D12_COMMAND_LIST_TYPE_COMPUTE;
     ID3D12Device *device;
     HRESULT hr;
 
@@ -870,14 +888,21 @@ bool init_compute_test_context_(unsigned int line, struct test_context *context)
     }
     device = context->device;
 
-    context->queue = create_command_queue_(line, device,
-            D3D12_COMMAND_LIST_TYPE_COMPUTE, D3D12_COMMAND_QUEUE_PRIORITY_NORMAL);
+#ifdef _WIN32
+    begin_renderdoc_capturing(device);
+    /* Workaround RenderDoc bug. It expects a DIRECT command queue to exist. */
+    if (renderdoc_api)
+        command_list_type = D3D12_COMMAND_LIST_TYPE_DIRECT;
+#endif
 
-    hr = ID3D12Device_CreateCommandAllocator(device, D3D12_COMMAND_LIST_TYPE_COMPUTE,
+    context->queue = create_command_queue_(line, device,
+            command_list_type, D3D12_COMMAND_QUEUE_PRIORITY_NORMAL);
+
+    hr = ID3D12Device_CreateCommandAllocator(device, command_list_type,
             &IID_ID3D12CommandAllocator, (void **)&context->allocator);
     ok_(line)(hr == S_OK, "Failed to create command allocator, hr %#x.\n", hr);
 
-    hr = ID3D12Device_CreateCommandList(device, 0, D3D12_COMMAND_LIST_TYPE_COMPUTE,
+    hr = ID3D12Device_CreateCommandList(device, 0, command_list_type,
             context->allocator, NULL, &IID_ID3D12GraphicsCommandList, (void **)&context->list);
     ok_(line)(hr == S_OK, "Failed to create command list, hr %#x.\n", hr);
 
