@@ -145,9 +145,11 @@ decl_test(test_resolve_query_data_in_reordered_command_list);
 decl_test(test_execute_indirect);
 decl_test(test_execute_indirect_state);
 decl_test(test_execute_indirect_state_predication);
+decl_test(test_execute_indirect_state_tier_11);
 decl_test(test_execute_indirect_multi_dispatch);
 decl_test(test_execute_indirect_multi_dispatch_root_constants);
 decl_test(test_execute_indirect_multi_dispatch_root_descriptors);
+decl_test(test_execute_indirect_state_vbo_offsets);
 decl_test(test_dispatch_zero_thread_groups);
 decl_test(test_unaligned_vertex_stride);
 decl_test(test_zero_vertex_stride);
@@ -156,7 +158,9 @@ decl_test(test_instance_id_dxil);
 decl_test(test_vertex_id_dxbc);
 decl_test(test_vertex_id_dxil);
 decl_test(test_copy_texture);
+decl_test(test_copy_texture_ds_edge_cases);
 decl_test(test_copy_texture_buffer);
+decl_test(test_copy_texture_bc_rgba);
 decl_test(test_copy_buffer_texture);
 decl_test(test_copy_block_compressed_texture);
 decl_test(test_copy_buffer_overlap);
@@ -387,6 +391,7 @@ decl_test(test_sampler_non_normalized_coordinates);
 decl_test(test_dynamic_depth_bias);
 decl_test(test_dynamic_index_strip_cut);
 decl_test(test_depth_bias_behaviour);
+decl_test(test_depth_bias_formats);
 decl_test(test_sampler_feedback_resource_creation);
 decl_test(test_sampler_feedback_format_features);
 decl_test(test_sampler_feedback_min_mip_level);
@@ -422,3 +427,9 @@ decl_test(test_workgraph_local_root_signature);
 decl_test(test_sm68_draw_parameters);
 decl_test(test_sm68_wave_size_range);
 decl_test(test_sm68_sample_cmp_bias_grad);
+decl_test(test_planar_video_formats);
+decl_test(test_vkd3d_dxvk_cmdbuf_interop);
+decl_test(test_large_texel_buffer_view);
+decl_test(test_destruction_notifier_callback);
+decl_test(test_destruction_notifier_interfaces);
+decl_test(test_sparse_default_mapping);
